-- -------------------------------------------------------------------------- --
--           _    _ ___   __           ____            _____      _           --
--          / \  | | \ \ / /__  _   _ / ___|__ _ _ __ | ____|__ _| |_         --
--         / _ \ | | |\ V / _ \| | | | |   / _` | '_ \|  _| / _` | __|        --
--        / ___ \| | | | | (_) | |_| | |__| (_| | | | | |__| (_| | |_         --
--       /_/   \_\_|_| |_|\___/ \__,_|\____\__,_|_| |_|_____\__,_|\__|        --
--                                                                            --
-- -------------------------------------------------------------------------- --

-- -------------------------------------------------------------------------- --
-- AllYouCanEat by kunda                                                      --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- AllYouCanEat is digging for details about the following statistics:        --
-- - 1774 = Different beverages consumed                                      --
-- - 1775 = Different foods eaten                                             --
-- - 1298 = Different bandage types used                                      --
-- - 1300 = Different health potions used                                     --
-- - 1302 = Different mana potions used                                       --
-- - 1304 = Different elixirs used                                            --
-- - 1306 = Different flasks consumed                                         --
--                                                                            --
-- With this AddOn it's easy to see which beverages, foods, bandage types,    --
-- potions, elixirs and flasks were consumed/used and which not.              --
-- The default statistic interface does not show such details.                --
--                                                                            --
-- The result is shown in a movable & resizable frame (accessible via LDB,    --
-- InterfaceOptions, '/allyoucaneat' or '/ayce'). If the statistic data is    --
-- saved to SavedVariables ('Data Manager' Button) it's possible to enable    --
-- an option to see this information in the various Tooltips or with other    --
-- characters.                                                                --
--                                                                            --
-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
-- !   It's impossible to consume everything because some items are class   ! --
-- !   dependent, profession dependent, faction dependent, quest rewards    ! --
-- !   (which you usually not get back if you finished the quest), some     ! --
-- !   items are only available via a TCG NPC and finally some items are    ! --
-- !   simply not available ingame.                                         ! --
-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
-- !!!  YES, this AddOn is absolutely USELESS, unless it's fun for you to !!! --
-- !!!  do a drink & food hunt across the whole WoW world :)              !!! --
-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
--                                                                            --
-- Notes:                                                                     --
-- -> The Tooltip server query code is may be a bit annoying, but this is     --
--    necessary to avoid disconnects, because some itemIDs that are returned  --
--    from the server in the initial statistic scan are invalid respectively  --
--    currently not available ingame.                                         --
--    This means: A Tooltip server query (mouseover an item in the MainFrame) --
--    via GameTooltip:SetHyperlink() is restricted to one server query every  --
--    5 seconds, for the same itemID to 10 seconds.                           --
--    - I never got a disconnect with this 2 intervals.                       --
--    - But just in case: A warning text is shown.                            --
--                                                                            --
-- -------------------------------------------------------------------------- --

-- ---------------------------------------------------------------------------------------------------------------------
AllYouCanEat_Options = {}                 -- SavedVariable options table
AllYouCanEat_Data = {}                    -- SavedVariable data table

local AllYouCanEat = CreateFrame("Frame") -- container
local L = AllYouCanEat_Locales            -- localization table

local GVAR = {}                           -- UI Widgets
local TEMPLATE = {}                       -- Templates

local AddonIcon = "Interface\\Minimap\\Tracking\\Food"

local _G = _G
local select = select
local math_floor = math.floor
local string_lower = string.lower
local string_find = string.find
local GetTime = GetTime
local GetAchievementNumCriteria = GetAchievementNumCriteria
local GetAchievementCriteriaInfo = GetAchievementCriteriaInfo
local GetContainerItemID = GetContainerItemID
local GetItemInfo = GetItemInfo
local mult2 = 10^2

local playerRealm              -- current realm name
local playerName               -- current player name
local playerGUID               -- current player GUID
local playerFaction            -- current player faction
local playerClass              -- current player class

local classcolors = {}
for class, color in pairs(RAID_CLASS_COLORS) do
	classcolors[class] = {r = color.r, g = color.g, b = color.b}
end

local DATA = {}                -- main data table
local GUI_MainDataGUID         -- current shown GUID data
local GUI_Data = {}            -- data table
local GUI_Data_Count           -- data table count
local GUI_SortBy = 1           -- data table sort by: 1 = number | 2 = itemId | 3 = itemLink | 4 = BopItem | 5 = QuestItem | 6 = ItemName
local GUI_SortOrder = 1        -- data table sort order
local GUI_Detail = 1           -- data table detail: 1 = all | 2 = consumed | 3 = not consumed
local GUI_Data_maxNum = 50     -- data table max rows
local GUI_Data_minNum = 5      -- data table min rows
local GUI_Data_curNum = 25     -- data table current rows
local SearchQuery

local GUI_Selection = 1774     -- default data
local GUI_scantime = ""        -- latest update

local GUI_CharListData = {}         -- character table
local GUI_CharListData_maxNum = 0   -- character table max rows

local myMAX_CHARACTERS_PER_REALM = 11

local StatisticID = {
	[1] = {id = 1774, name = L["Beverages"],      icon = "Interface\\Icons\\INV_Drink_15"},        -- StatisticID 1774 = Different beverages consumed
	[2] = {id = 1775, name = L["Foods"],          icon = "Interface\\Icons\\INV_Misc_Food_08"},    -- StatisticID 1775 = Different foods eaten
	[3] = {id = 1298, name = L["Bandage"],        icon = "Interface\\Icons\\INV_Misc_Bandage_14"}, -- StatisticID 1298 = Different bandage types used
	[4] = {id = 1300, name = L["Health Potions"], icon = "Interface\\Icons\\INV_Potion_51"},       -- StatisticID 1300 = Different health potions used
	[5] = {id = 1302, name = L["Mana Potions"],   icon = "Interface\\Icons\\INV_Potion_72"},       -- StatisticID 1302 = Different mana potions used
	[6] = {id = 1304, name = L["Elixirs"],        icon = "Interface\\Icons\\INV_Potion_66"},       -- StatisticID 1304 = Different elixirs used
	[7] = {id = 1306, name = L["Flasks"],         icon = "Interface\\Icons\\INV_Potion_90"},       -- StatisticID 1306 = Different flasks consumed
}

local itemQuality = {
	[1] = {name = ITEM_QUALITY0_DESC, color = "9d9d9d"}, -- 0 | poor
	[2] = {name = ITEM_QUALITY1_DESC, color = "ffffff"}, -- 1 | common
	[3] = {name = ITEM_QUALITY2_DESC, color = "1eff00"}, -- 2 | uncommon
	[4] = {name = ITEM_QUALITY3_DESC, color = "0070dd"}, -- 3 | rare
	[5] = {name = ITEM_QUALITY4_DESC, color = "a335ee"}, -- 4 | epic
	[6] = {name = ITEM_QUALITY5_DESC, color = "ff8000"}, -- 5 | legendary
	[7] = {name = ITEM_QUALITY6_DESC, color = "e6cc80"}, -- 6 | artifact (not in game)
	[8] = {name = ITEM_ACCOUNTBOUND,  color = "e6cc80"}, -- 7 | account bound
	[9] = {name = UNKNOWN,            color = "666666"}  -- 8 | unknown (item that is not in local cache or currently not availabe)
}

local TooltipCodeIsNeeded
local ServerQueryTime = {}            -- checker for Tooltip server query in function AllYouCanEat:ShowTooltip() to avoid disconnects
local ServerQueryTimeLast = GetTime() -- checker for Tooltip server query in function AllYouCanEat:ShowTooltip() to avoid disconnects

local criteriaTimer = ServerQueryTimeLast
local UPDATE_itemID -- value for CRITERIA_UPDATE
local UPDATE_statID -- value for CRITERIA_UPDATE
local UPDATE_catID  -- value for CRITERIA_UPDATE

local months = {CalendarGetMonthNames()}

local size = {}
size.W_MainFrame = 420
size.W_MinMainFrame = 420
size.W_MaxMainFrame = 600
size.H_Headline = 28
size.H_Status = 20

size.H_SelectionCharList = 20
size.H_SelectionStat = 32+2+10+2+10 --56
size.H_SelectionButtonAll = 20
size.W_SelectionButtonSpace = 6

size.H_SelectionAllNormal = 4 + size.H_SelectionCharList + 4 + size.H_SelectionStat + 4 + size.H_SelectionButtonAll-1

size.H_DataCur = 3+(1*12)+2+(GUI_Data_curNum*12)+3 -- (GUI_Data_curNum 25) = 320
size.H_DataMin = 3+(1*12)+2+(GUI_Data_minNum*12)+3 -- (GUI_Data_minNum 5)  =  80
size.H_DataMax = 3+(1*12)+2+(GUI_Data_maxNum*12)+3 -- (GUI_Data_maxNum 50) = 620

size.H_MainframeNormal    = (size.H_Headline-1) + size.H_SelectionAllNormal + (size.H_DataCur-1) + (size.H_Status-1) -- 27 + 87 + 320 + 19 = 393
size.H_MainframeMinNormal = (size.H_Headline-1) + size.H_SelectionAllNormal + (size.H_DataMin-1) + (size.H_Status-1) -- 27 + 87 +  80 + 19 = 213
size.H_MainframeMaxNormal = (size.H_Headline-1) + size.H_SelectionAllNormal + (size.H_DataMax-1) + (size.H_Status-1) -- 27 + 87 + 620 + 19 = 753

size.H_charmanagerScrollFrame = 0

local Hooks = {}
function Hooks:SetAction(id)
	local _, item = self:GetItem()
	if not item then return end
	AllYouCanEat:AddNote(self, item)
end
function Hooks:SetAuctionItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetAuctionSellItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetBagItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetHyperlink()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetInboxItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetInventoryItem(unit, slot)
	if type(slot) ~= "number" or slot < 0 then return end
	AllYouCanEat:AddNote(self)
end
function Hooks:SetLootItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetLootRollItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetMerchantCostItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetMerchantItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetBuybackItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetQuestItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetQuestLogItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetSendMailItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetTradePlayerItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetTradeSkillItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetTradeTargetItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetGuildBankItem()
	AllYouCanEat:AddNote(self)
end

local function NOOP() end

local function InstallHooks(tooltip, hooks)
	for name, func in pairs(hooks) do
		hooksecurefunc(tooltip, name, func)
	end
end

local function yearmonthday()
	local YYYY    = date("%Y")
	local MM      = date("%m")
	local Mstring = months[tonumber(MM)]
	local DD      = date("%d")
	return YYYY.."-"..Mstring.."-"..DD
end

local function UpdateTime()
	local hour, minute = GetGameTime()
	if hour < 10 then
		hour = "0"..hour
	end
	if minute < 10 then
		minute = "0"..minute
	end
	GUI_scantime = hour..":"..minute
	
	if AllYouCanEat_Data.chars[playerRealm][playerGUID].savestats then
		AllYouCanEat_Data.chars[playerRealm][playerGUID].update = GUI_scantime.." | "..yearmonthday()
	else
		AllYouCanEat_Data.chars[playerRealm][playerGUID].update = nil
	end
end

local function ClassHexColor(class)
	local hex
	if classcolors[class] then
		hex = string.format("%.2x%.2x%.2x", classcolors[class].r*255, classcolors[class].g*255, classcolors[class].b*255)
	end
	return hex or "cccccc"
end

local function Print(...)
	print("|cffffff7fAllYouCanEat:|r", ...)
end

local function Desaturation(texture, desaturation)
	local shaderSupported = texture:SetDesaturated(desaturation)
	if not shaderSupported then
		if desaturation then
			texture:SetVertexColor(0.5, 0.5, 0.5)
		else
			texture:SetVertexColor(1.0, 1.0, 1.0)
		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
-- Template BorderTRBL START ----------------------------------------
TEMPLATE.BorderTRBL = function(frame, color) -- TRBL = Top-Right-Bottom-Left
	frame.FrameBorder = frame:CreateTexture(nil, "BORDER")
	frame.FrameBorder:SetPoint("TOPLEFT", 1, -1)
	frame.FrameBorder:SetPoint("BOTTOMRIGHT", -1, 1)
	frame.FrameBorder:SetTexture(0, 0, 0, 1)
	frame.FrameBackground = frame:CreateTexture(nil, "BACKGROUND")
	frame.FrameBackground:SetPoint("TOPLEFT", 0, 0)
	frame.FrameBackground:SetPoint("BOTTOMRIGHT", 0, 0)
	frame.FrameBackground:SetTexture(0.4, 0.4, 0.4, 1)
end
-- Template BorderTRBL END ----------------------------------------

-- Template TextButton START ----------------------------------------
TEMPLATE.DisableTextButton = function(button)
	button.Border:SetTexture(0.4, 0.4, 0.4, 1)
	button:Disable()
end

TEMPLATE.EnableTextButton = function(button, action)
	local buttoncolor
	if action == 1 then
		bordercolor = {0.73, 0.26, 0.21, 1}
	elseif action == 2 then
		bordercolor = {0.43, 0.32, 0.68, 1}
	elseif action == 3 then
		bordercolor = {0.24, 0.46, 0.21, 1}
	elseif action == 4 then
		bordercolor = {0.73, 0.26, 0.21, 1}
	else
		bordercolor = {1, 1, 1, 1}
	end
	button.Border:SetTexture(bordercolor[1], bordercolor[2], bordercolor[3], bordercolor[4])
	button:Enable()
end

TEMPLATE.TextButton = function(button, text, action)
	local buttoncolor
	local bordercolor
	if action == 1 then
		button:SetNormalFontObject("GameFontNormal")
		button:SetDisabledFontObject("GameFontDisable") 
		buttoncolor = {0.38, 0, 0, 1}
		bordercolor = {0.73, 0.26, 0.21, 1}
	elseif action == 2 then
		button:SetNormalFontObject("GameFontNormalSmall")
		button:SetDisabledFontObject("GameFontDisableSmall")
		buttoncolor = {0, 0, 0.5, 1}
		bordercolor = {0.43, 0.32, 0.68, 1}
	elseif action == 3 then
		button:SetNormalFontObject("GameFontNormalSmall")
		button:SetDisabledFontObject("GameFontDisableSmall")
		buttoncolor = {0, 0.2, 0, 1}
		bordercolor = {0.24, 0.46, 0.21, 1}
	elseif action == 4 then
		button:SetNormalFontObject("GameFontNormalSmall")
		button:SetDisabledFontObject("GameFontDisableSmall")
		buttoncolor = {0.38, 0, 0, 1}
		bordercolor = {0.73, 0.26, 0.21, 1}
	else
		button:SetNormalFontObject("GameFontNormal")
		button:SetDisabledFontObject("GameFontDisable")
		buttoncolor = {0, 0, 0, 1}
		bordercolor = {1, 1, 1, 1}
	end

	button.Background = button:CreateTexture(nil, "BORDER")
	button.Background:SetPoint("TOPLEFT", 1, -1)
	button.Background:SetPoint("BOTTOMRIGHT", -1, 1)
	button.Background:SetTexture(0, 0, 0, 1)

	button.Border = button:CreateTexture(nil, "BACKGROUND")
	button.Border:SetPoint("TOPLEFT", 0, 0)
	button.Border:SetPoint("BOTTOMRIGHT", 0, 0)
	button.Border:SetTexture(bordercolor[1], bordercolor[2], bordercolor[3], bordercolor[4])

	button.Normal = button:CreateTexture(nil, "ARTWORK")
	button.Normal:SetPoint("TOPLEFT", 2, -2)
	button.Normal:SetPoint("BOTTOMRIGHT", -2, 2)
	button.Normal:SetTexture(buttoncolor[1], buttoncolor[2], buttoncolor[3], buttoncolor[4])
	button:SetNormalTexture(button.Normal)

	button.Disabled = button:CreateTexture(nil, "OVERLAY")
	button.Disabled:SetPoint("TOPLEFT", 3, -3)
	button.Disabled:SetPoint("BOTTOMRIGHT", -3, 3)
	button.Disabled:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetDisabledTexture(button.Disabled)

	button.Highlight = button:CreateTexture(nil, "OVERLAY")
	button.Highlight:SetPoint("TOPLEFT", 3, -3)
	button.Highlight:SetPoint("BOTTOMRIGHT", -3, 3)
	button.Highlight:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetHighlightTexture(button.Highlight)

	button:SetPushedTextOffset(1, -1)	
	button:SetText(text)
end
-- Template TextButton END ----------------------------------------

-- Template IconButton START ----------------------------------------
TEMPLATE.IconButton = function(button, cut)
	button.Back = button:CreateTexture(nil, "BORDER")
	button.Back:SetPoint("TOPLEFT", 1, -1)
	button.Back:SetPoint("BOTTOMRIGHT", -1, 1)
	button.Back:SetTexture(0, 0, 0, 1)

	button.Border = button:CreateTexture(nil, "BACKGROUND")
	button.Border:SetPoint("TOPLEFT", 0, 0)
	button.Border:SetPoint("BOTTOMRIGHT", 0, 0)
	button.Border:SetTexture(0.4, 0.4, 0.4, 1)

	if cut == 1 then
		button.Normal = button:CreateTexture(nil, "ARTWORK")
		button.Normal:SetPoint("TOPLEFT", 3, -3)
		button.Normal:SetPoint("BOTTOMRIGHT", -3, 3)
		button.Normal:SetTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
		button.Normal:SetTexCoord(8/32, 22/32, 10/32, 22/32)
		button:SetNormalTexture(button.Normal)
		button.Push = button:CreateTexture(nil, "ARTWORK")
		button.Push:SetPoint("TOPLEFT", 4, -4)
		button.Push:SetPoint("BOTTOMRIGHT", -4, 4)
		button.Push:SetTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
		button.Push:SetTexCoord(8/32, 22/32, 10/32, 22/32)
		button:SetPushedTexture(button.Push)
	elseif cut == 2 then
		button.Normal = button:CreateTexture(nil, "ARTWORK")
		button.Normal:SetPoint("TOPLEFT", 3, -3)
		button.Normal:SetPoint("BOTTOMRIGHT", -3, 3)
		button.Normal:SetTexture("Interface\\GossipFrame\\BinderGossipIcon")
		button:SetNormalTexture(button.Normal)
		button.Push = button:CreateTexture(nil, "ARTWORK")
		button.Push:SetPoint("TOPLEFT", 4, -4)
		button.Push:SetPoint("BOTTOMRIGHT", -4, 4)
		button.Push:SetTexture("Interface\\GossipFrame\\BinderGossipIcon")
		button:SetPushedTexture(button.Push)
	end

	button.Highlight = button:CreateTexture(nil, "OVERLAY")
	button.Highlight:SetPoint("TOPLEFT", 3, -3)
	button.Highlight:SetPoint("BOTTOMRIGHT", -3, 3)
	button.Highlight:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetHighlightTexture(button.Highlight)
end
-- Template IconButton END ----------------------------------------

-- Template CheckButton START ----------------------------------------
TEMPLATE.DisableCheckButton = function(button)
	button.Text:SetTextColor(0.5, 0.5, 0.5)
	button.Border:SetTexture(0.4, 0.4, 0.4, 1)
	button:Disable()
end

TEMPLATE.EnableCheckButton = function(button)
	button.Text:SetTextColor(1, 1, 1)
	button.Border:SetTexture(0.8, 0.2, 0.2, 1)
	button:Enable()
end

TEMPLATE.CheckButton = function(button, size, space, text)
	button.Border = button:CreateTexture(nil, "BACKGROUND")
	button.Border:SetWidth( size )
	button.Border:SetHeight( size )
	button.Border:SetPoint("LEFT", 0, 0)
	button.Border:SetTexture(0.4, 0.4, 0.4, 1)

	button.Background = button:CreateTexture(nil, "BORDER")
	button.Background:SetPoint("TOPLEFT", button.Border, "TOPLEFT", 1, -1)
	button.Background:SetPoint("BOTTOMRIGHT", button.Border, "BOTTOMRIGHT", -1, 1)
	button.Background:SetTexture(0, 0, 0, 1)

	button.Normal = button:CreateTexture(nil, "ARTWORK")
	button.Normal:SetPoint("TOPLEFT", button.Border, "TOPLEFT", 1, -1)
	button.Normal:SetPoint("BOTTOMRIGHT", button.Border, "BOTTOMRIGHT", -1, 1)
	button.Normal:SetTexture(0, 0, 0, 1)
	button:SetNormalTexture(button.Normal)

	button.Push = button:CreateTexture(nil, "ARTWORK")
	button.Push:SetPoint("TOPLEFT", button.Border, "TOPLEFT", 4, -4)
	button.Push:SetPoint("BOTTOMRIGHT", button.Border, "BOTTOMRIGHT", -4, 4)
	button.Push:SetTexture(0.4, 0.4, 0.4, 0.5)
	button:SetPushedTexture(button.Push)

	button.Disabled = button:CreateTexture(nil, "ARTWORK")
	button.Disabled:SetPoint("TOPLEFT", button.Border, "TOPLEFT", 3, -3)
	button.Disabled:SetPoint("BOTTOMRIGHT", button.Border, "BOTTOMRIGHT", -3, 3)
	button.Disabled:SetTexture(0.4, 0.4, 0.4, 0.5)
	button:SetDisabledTexture(button.Disabled)

	button.Checked = button:CreateTexture(nil, "ARTWORK")
	button.Checked:SetWidth( size )
	button.Checked:SetHeight( size )
	button.Checked:SetPoint("LEFT", 0, 0)
	button.Checked:SetTexture("Interface\\Buttons\\UI-CheckBox-Check")
	button:SetCheckedTexture(button.Checked)

	button.Text = button:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
	button.Text:SetHeight( size )
	button.Text:SetPoint("LEFT", button.Normal, "RIGHT", space, 0)
	button.Text:SetJustifyH("LEFT")
	button.Text:SetText(text)
	button.Text:SetTextColor(1, 1, 1, 1)

	button:SetWidth(size + space + button.Text:GetStringWidth() + space)
	button:SetHeight(size)

	button.Highlight = button:CreateTexture(nil, "OVERLAY")
	button.Highlight:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
	button.Highlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
	button.Highlight:SetTexture(1, 1, 1, 0.1)
	button.Highlight:Hide()

	button:SetScript("OnEnter", function() button.Highlight:Show() end)
	button:SetScript("OnLeave", function() button.Highlight:Hide() end)
end
-- Template CheckButton END ----------------------------------------

-- Template TabButton START ----------------------------------------
TEMPLATE.SetTabButton = function(button, showIt)
	if showIt then
		button.TextureBottom:SetTexture(0, 0, 0, 1)
		button.TextureBorder:SetTexture(0.4, 0.4, 0.4, 1)
	else
		button.TextureBottom:SetTexture(0.4, 0.4, 0.4, 1)
		button.TextureBorder:SetTexture(0.4, 0.4, 0.4, 0.4)
	end
end

TEMPLATE.TabButton = function(button, text, detail)
	button.Texture = button:CreateTexture(nil, "BORDER")
	button.Texture:SetPoint("TOPLEFT", 1, -1)
	button.Texture:SetPoint("BOTTOMRIGHT", -1, 1)
	button.Texture:SetTexture(0, 0, 0, 1)

	button.TextureBorder = button:CreateTexture(nil, "BACKGROUND")
	button.TextureBorder:SetPoint("TOPLEFT", 0, 0)
	button.TextureBorder:SetPoint("BOTTOMRIGHT", -1, 1)
	button.TextureBorder:SetPoint("TOPRIGHT"    ,0, 0)
	button.TextureBorder:SetPoint("BOTTOMLEFT" ,1, 1)
	button.TextureBorder:SetTexture(0.4, 0.4, 0.4, 1)

	button.TextureBottom = button:CreateTexture(nil, "ARTWORK")
	button.TextureBottom:SetPoint("TOPLEFT", button, "BOTTOMLEFT" ,1, 2)
	button.TextureBottom:SetPoint("BOTTOMLEFT"  ,1, 1)
	button.TextureBottom:SetPoint("TOPRIGHT", button, "BOTTOMRIGHT" ,-1, 2)
	button.TextureBottom:SetPoint("BOTTOMRIGHT" ,-1, 1)
	button.TextureBottom:SetTexture(0.4, 0.4, 0.4, 1)

	button.TextureHighlight = button:CreateTexture(nil, "ARTWORK")
	button.TextureHighlight:SetPoint("TOPLEFT", 3, -3)
	button.TextureHighlight:SetPoint("BOTTOMRIGHT", -3, 3)
	button.TextureHighlight:SetTexture(1, 1, 1, 0.2)
	button:SetHighlightTexture(button.TextureHighlight)

	button.Text = button:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
	button.Text:SetText(text)
	button.Text:SetWidth( button.Text:GetStringWidth()+10 )
	button.Text:SetHeight(12)
	button.Text:SetPoint("CENTER", button, "CENTER", 0, 0)
	button.Text:SetJustifyH("CENTER")

	if detail == 1 then
		button.Text:SetTextColor(1, 1, 1, 1)
	elseif detail == 2 then
		button.Text:SetTextColor(0, 0.75, 0, 1)
	elseif detail == 3 then
		button.Text:SetTextColor(1, 0, 0, 1)
	end
end
-- Template TabButton END ----------------------------------------

-- Template EditBox START ----------------------------------------
TEMPLATE.EditBox = function(button, width, showIcon, funcSearch, funcClear)
	local fxWidth = 17
	if not showIcon then
		fxWidth = 0.00000001
	end
	local boxwidth = width-(fxWidth+2+2+17)

	button.SearchIcon = button:CreateTexture(nil, "ARTWORK")
	button.SearchIcon:SetWidth(fxWidth)
	button.SearchIcon:SetHeight(fxWidth)
	button.SearchIcon:SetPoint("LEFT", 0, 0)
	button.SearchIcon:SetTexture("Interface\\Minimap\\Tracking\\None")
	button.SearchIcon:SetTexCoord(3/32, 31/32, 1/32, 29/32)

	button.EditBox = CreateFrame("EditBox", nil, button)
	button.EditBox:SetWidth(boxwidth)
	button.EditBox:SetHeight(17)
	button.EditBox:SetPoint("LEFT", button.SearchIcon, "RIGHT", 2, 0)
	button.EditBox:SetAutoFocus(false)
	button.EditBox:SetFontObject(GameFontNormalSmall)
	button.EditBox:SetTextColor(1, 1, 1, 1)
	button.EditBox:SetTextInsets(2, 0, 0, 0) 

	button.EditBoxLeft = button.EditBox:CreateTexture(nil, "BACKGROUND")
	button.EditBoxLeft:SetWidth(5)
	button.EditBoxLeft:SetHeight(17)
	button.EditBoxLeft:SetPoint("LEFT", 0, 0)
	button.EditBoxLeft:SetTexture("Interface\\Chatframe\\UI-ChatInputBorderFocus-Left")
	button.EditBoxLeft:SetTexCoord(8/32, 13/32, 8/32, 25/32)
	button.EditBoxLeft:SetVertexColor(0.6, 0.6, 0.6, 1)

	button.EditBoxMiddle = button.EditBox:CreateTexture(nil, "BACKGROUND")
	button.EditBoxMiddle:SetWidth(boxwidth-5-5)
	button.EditBoxMiddle:SetHeight(17)
	button.EditBoxMiddle:SetPoint("LEFT", button.EditBoxLeft, "RIGHT", 0, 0)
	button.EditBoxMiddle:SetTexture("Interface\\Chatframe\\UI-ChatInputBorderFocus-Mid")
	button.EditBoxMiddle:SetTexCoord(0/32, 1/32, 8/32, 25/32)
	button.EditBoxMiddle:SetVertexColor(0.6, 0.6, 0.6, 1)

	button.EditBoxRight = button.EditBox:CreateTexture(nil, "BACKGROUND")
	button.EditBoxRight:SetWidth(5)
	button.EditBoxRight:SetHeight(17)
	button.EditBoxRight:SetPoint("LEFT", button.EditBoxMiddle, "RIGHT", 0, 0)
	button.EditBoxRight:SetTexture("Interface\\Chatframe\\UI-ChatInputBorderFocus-Right")
	button.EditBoxRight:SetTexCoord(19/32, 24/32, 8/32, 25/32)
	button.EditBoxRight:SetVertexColor(0.6, 0.6, 0.6, 1)

	button.EditBoxClear = CreateFrame("Button", nil, button)
	button.EditBoxClear:SetWidth(17)
	button.EditBoxClear:SetHeight(17)
	button.EditBoxClear:SetPoint("LEFT", button.EditBox, "RIGHT", 2, 0)

	button.EditBoxClearNormal = button.EditBoxClear:CreateTexture(nil, "ARTWORK")
	button.EditBoxClearNormal:SetPoint("TOPLEFT", 0, 0)
	button.EditBoxClearNormal:SetPoint("BOTTOMRIGHT", 0, 0)
	button.EditBoxClearNormal:SetAlpha(0.75)
	button.EditBoxClearNormal:SetTexture("Interface\\FriendsFrame\\ClearBroadcastIcon")
	button.EditBoxClear:SetNormalTexture(button.EditBoxClearNormal)

	button.EditBoxClearDisable = button.EditBoxClear:CreateTexture(nil, "ARTWORK")
	button.EditBoxClearDisable:SetPoint("TOPLEFT", 0, 0)
	button.EditBoxClearDisable:SetPoint("BOTTOMRIGHT", 0, 0)
	button.EditBoxClearDisable:SetAlpha(0.25)
	button.EditBoxClearDisable:SetTexture("Interface\\FriendsFrame\\ClearBroadcastIcon")
	button.EditBoxClear:SetDisabledTexture(button.EditBoxClearDisable)

	Desaturation(button.EditBoxClearDisable, true)

	button.EditBox:SetScript("OnShow",          function(self) self:SetAutoFocus(false) end)
	button.EditBox:SetScript("OnEnterPressed",  function(self) self:ClearFocus() end)
	button.EditBox:SetScript("OnEscapePressed", function(self) self:ClearFocus() end)
	button.EditBox:SetScript("OnTextChanged",   function() funcSearch() end)

	button.EditBoxClear:Disable()
	button.EditBoxClear:SetScript("OnClick", function() funcClear() end)
	button.EditBoxClear:SetScript("OnMouseDown", function()
		button.EditBoxClearNormal:SetPoint("TOPLEFT", 1, -1)
		button.EditBoxClearNormal:SetPoint("BOTTOMRIGHT", -1, 1)
	end)
	button.EditBoxClear:SetScript("OnMouseUp", function()
		button.EditBoxClearNormal:SetPoint("TOPLEFT", 0, 0)
		button.EditBoxClearNormal:SetPoint("BOTTOMRIGHT", 0, 0)
	end)
	button.EditBoxClear:SetScript("OnEnter", function()
		button.EditBoxClearNormal:SetAlpha(1)
	end)
	button.EditBoxClear:SetScript("OnLeave", function()
		button.EditBoxClearNormal:SetAlpha(0.75)
	end)
end
-- Template EditBox END ----------------------------------------

-- Template ScrollFrame START ----------------------------------------
TEMPLATE.ButtonTextureScrollFrame = function(button, pos)
	local textureNormal = button:CreateTexture(nil, "ARTWORK")
	textureNormal:SetWidth(16)
	textureNormal:SetHeight(16)
	textureNormal:SetPoint("LEFT", 0, 0)
	textureNormal:SetTexture("Interface\\Buttons\\UI-ScrollBar-Scroll"..pos.."Button-Up")
	textureNormal:SetTexCoord(6/32, 25/32, 7/32, 24/32)
	button:SetNormalTexture(textureNormal)

	local texturePush = button:CreateTexture(nil, "ARTWORK")
	texturePush:SetWidth(16)
	texturePush:SetHeight(16)
	texturePush:SetPoint("TOPLEFT", 1, -1)
	texturePush:SetPoint("BOTTOMRIGHT", -1, 1)
	texturePush:SetTexture("Interface\\Buttons\\UI-ScrollBar-Scroll"..pos.."Button-Down")
	texturePush:SetTexCoord(6/32, 25/32, 7/32, 24/32)
	button:SetPushedTexture(texturePush)

	local textureHighlight = button:CreateTexture(nil, "ARTWORK")
	textureHighlight:SetWidth(16)
	textureHighlight:SetHeight(16)
	textureHighlight:SetPoint("LEFT", 0, 0)
	textureHighlight:SetTexture("Interface\\Buttons\\UI-ScrollBar-Scroll"..pos.."Button-Highlight")
	textureHighlight:SetTexCoord(6/32, 25/32, 7/32, 24/32)
	button:SetHighlightTexture(textureHighlight)

	local textureDisable = button:CreateTexture(nil, "ARTWORK")
	textureDisable:SetWidth(16)
	textureDisable:SetHeight(16)
	textureDisable:SetPoint("LEFT", 0, 0)
	textureDisable:SetTexture("Interface\\Buttons\\UI-ScrollBar-Scroll"..pos.."Button-Disabled")
	textureDisable:SetTexCoord(6/32, 25/32, 7/32, 24/32)
	button:SetDisabledTexture(textureDisable)
end

TEMPLATE.ScrollFrame = function(frame, child)
	frame.offset = 0
	frame:EnableMouseWheel(true)

	frame.ScrollBar = CreateFrame("Slider", nil, frame)
	frame.ScrollBar:SetWidth(16)
	frame.ScrollBar:SetHeight(0)
	frame.ScrollBar:SetPoint("TOPLEFT", frame, "TOPRIGHT", 2, -16)
	frame.ScrollBar:SetPoint("BOTTOMLEFT", frame, "BOTTOMRIGHT", 2, 16)
	frame.ScrollBar:SetMinMaxValues(0, 0)
	frame.ScrollBar:SetValue(0)
	frame.ScrollBar:SetOrientation("VERTICAL")
	frame.ScrollBar:EnableMouseWheel(true)
	frame.ScrollBar:SetScript("OnValueChanged", function(self, value)
		frame:SetVerticalScroll(value)
	end)
	frame.ScrollBar:SetScript("OnMouseWheel", function(self, delta)
		if delta > 0 then
			frame.ScrollBar:SetValue(frame.ScrollBar:GetValue() - (frame.ScrollBar:GetHeight() / 2))
		else
			frame.ScrollBar:SetValue(frame.ScrollBar:GetValue() + (frame.ScrollBar:GetHeight() / 2))
		end
	end)

	frame.ScrollBG = frame.ScrollBar:CreateTexture(nil, "BACKGROUND")
	frame.ScrollBG:SetAllPoints(frame.ScrollBar)
	frame.ScrollBG:SetTexture(0, 0, 0, 0)

	frame.ScrollBarScrollUpButton = CreateFrame("Button", nil, frame.ScrollBar)
	frame.ScrollBarScrollUpButton:SetWidth(16)
	frame.ScrollBarScrollUpButton:SetHeight(16)
	frame.ScrollBarScrollUpButton:SetPoint("BOTTOM", frame.ScrollBar, "TOP", 0, 0)
	frame.ScrollBarScrollUpButton:SetScript("OnClick", function()
		frame.ScrollBar:SetValue(frame.ScrollBar:GetValue() - (frame.ScrollBar:GetHeight() / 2))
		PlaySound("UChatScrollButton")
	end)
	TEMPLATE.ButtonTextureScrollFrame(frame.ScrollBarScrollUpButton, "Up")

	frame.ScrollBarScrollDownButton = CreateFrame("Button", nil, frame.ScrollBar)
	frame.ScrollBarScrollDownButton:SetWidth(16)
	frame.ScrollBarScrollDownButton:SetHeight(16)
	frame.ScrollBarScrollDownButton:SetPoint("TOP", frame.ScrollBar, "BOTTOM", 0, 0)
	frame.ScrollBarScrollDownButton:SetScript("OnClick", function()
		frame.ScrollBar:SetValue(frame.ScrollBar:GetValue() + (frame.ScrollBar:GetHeight() / 2))
		PlaySound("UChatScrollButton")
	end)
	TEMPLATE.ButtonTextureScrollFrame(frame.ScrollBarScrollDownButton, "Down")

	frame.ScrollBarThumbTexture = frame.ScrollBar:CreateTexture(nil, "BORDER")
	frame.ScrollBarThumbTexture:SetWidth(14)
	frame.ScrollBarThumbTexture:SetHeight(16)
	frame.ScrollBarThumbTexture:SetTexture("Interface\\Buttons\\UI-ScrollBar-Knob")
	frame.ScrollBarThumbTexture:SetTexCoord(8/32, 23/32, 8/32, 23/32)
	frame.ScrollBar:SetThumbTexture(frame.ScrollBarThumbTexture)

	frame:SetScript("OnScrollRangeChanged", function(self, xrange, yrange)
		if not yrange then
			yrange = self:GetVerticalScrollRange()
		end
		local value = frame.ScrollBar:GetValue()
		if value > yrange then
			value = yrange
		end
		frame.ScrollBar:SetMinMaxValues(0, yrange)
		frame.ScrollBar:SetValue(value)
		if math_floor(yrange) == 0 then
			frame.ScrollBarScrollDownButton:Disable()
			frame.ScrollBarScrollUpButton:Disable()
			frame.ScrollBarScrollDownButton:Show()
			frame.ScrollBarScrollUpButton:Show()
		else
			frame.ScrollBarScrollDownButton:Show()
			frame.ScrollBarScrollUpButton:Show()
			frame.ScrollBar:Show()
			-- The 0.005 is to account for precision errors
			if yrange - value > 0.005 then
				frame.ScrollBarScrollDownButton:Enable()
			else
				frame.ScrollBarScrollDownButton:Disable()
			end
		end
	end)
	frame:SetScript("OnMouseWheel", function(self, delta)
		if delta > 0 then
			frame.ScrollBar:SetValue(frame.ScrollBar:GetValue() - (frame.ScrollBar:GetHeight() / 2))
		else
			frame.ScrollBar:SetValue(frame.ScrollBar:GetValue() + (frame.ScrollBar:GetHeight() / 2))
		end
	end)
	frame:SetScript("OnVerticalScroll", function(self, off)
		frame.ScrollBar:SetValue(off)
	end)

	if child then
		frame.ScrollChildFrame = CreateFrame("Frame", nil, frame)
		frame.ScrollChildFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 0)
		frame.ScrollChildFrame:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 0, 0)
		frame.ScrollChildFrame:SetHeight(1)
		frame.ScrollChildFrame:SetWidth(1)
		--frame:SetScrollChild(frame.ScrollChildFrame)
	end
end
-- Template ScrollFrame END ----------------------------------------

-- Template DropDownBox START ----------------------------------------
TEMPLATE.DropDownBox = function(button, text)
	button.Normal = button:CreateTexture(nil, "ARTWORK")
	button.Normal:SetHeight(14/1.25)
	button.Normal:SetWidth(15/1.25)
	button.Normal:SetPoint("RIGHT", -4, 0)
	button.Normal:SetTexture("Interface\\Buttons\\UI-Panel-ExpandButton-Up")
	button.Normal:SetTexCoord(8/32, 23/32, 9/32, 23/32)
	button:SetNormalTexture(button.Normal)

	button.Highlight = button:CreateTexture(nil, "ARTWORK")
	button.Highlight:SetPoint("TOPLEFT", 1, -1)
	button.Highlight:SetPoint("BOTTOMRIGHT", -1, 1)
	button.Highlight:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetHighlightTexture(button.Highlight)

	button.Text = button:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
	button.Text:SetWidth(140)
	button.Text:SetHeight(20)
	button.Text:SetPoint("LEFT", button, "LEFT", 4, 0)
	button.Text:SetTextColor(1, 1, 1, 1)
	button.Text:SetJustifyH("LEFT")
end
-- Template DropDownBox END ----------------------------------------

-- Template SortPosition START ----------------------------------------
TEMPLATE.SortPosition = function(button, text, pos, width, dummy)
	button.Background = button:CreateTexture(nil, "BACKGROUND")
	button.Background:SetPoint("TOPLEFT", 0, 0)
	button.Background:SetPoint("BOTTOMRIGHT", 0, 0)
	button.Background:SetTexture(1, 1, 1, 0.1)

	local buttontext
	if text then
		if pos == "right" then
			buttontext = button:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
			buttontext:SetText(text)
			local fw = buttontext:GetStringWidth()
			if fw >= width then
				buttontext:SetWidth( width-10 )
			end
			buttontext:SetHeight(12)
			buttontext:SetPoint("RIGHT", button, "RIGHT", 0, 0)
			buttontext:SetJustifyH("RIGHT")
		else
			buttontext = button:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
			buttontext:SetText(text)
			local fw = buttontext:GetStringWidth()
			if fw >= width then
				buttontext:SetWidth( width-10 )
			end
			buttontext:SetHeight(12)
			buttontext:SetPoint("LEFT", button, "LEFT", 0, 0)
			buttontext:SetJustifyH("LEFT")
		end
	end

	if dummy then return end

	button.arrow = button:CreateTexture(nil, "BACKGROUND")
	button.arrow:SetWidth(7)
	button.arrow:SetHeight(8)
	button.arrow:SetTexture("Interface\\Buttons\\UI-SortArrow")
	if text then
		if pos == "right" then
			button.arrow:SetPoint("RIGHT", buttontext, "LEFT", -1, 0)
		else
			button.arrow:SetPoint("LEFT", buttontext, "RIGHT", 1, 0)
		end
	else
		button.arrow:SetPoint("CENTER", button, "CENTER", 0, 0)
	end

	button.Highlight = button:CreateTexture(nil, "ARTWORK")
	button.Highlight:SetPoint("TOPLEFT", 0, 0)
	button.Highlight:SetPoint("BOTTOMRIGHT", 0, 0)
	button.Highlight:SetTexture(1, 1, 1, 0.2)
	button:SetHighlightTexture(button.Highlight)
end
-- Template SortPosition END ----------------------------------------

-- Template DataList START ----------------------------------------
TEMPLATE.DataList = function(button, isSort)
	if isSort then
		button.NumberSort = CreateFrame("Button", nil, button)
		TEMPLATE.SortPosition(button.NumberSort, L["No."], "right", 40)
		button.NumberSort:SetWidth(40)
		button.NumberSort:SetHeight(12)
		button.NumberSort:SetPoint("LEFT", button, "LEFT", 0, 0)
		button.NumberSort:SetScript("OnClick", function() AllYouCanEat:GUIsort(1) end)

		button.ItemIdSort = CreateFrame("Button", nil, button)
		TEMPLATE.SortPosition(button.ItemIdSort, L["ItemID"], "right", 60)
		button.ItemIdSort:SetWidth(60)
		button.ItemIdSort:SetHeight(12)
		button.ItemIdSort:SetPoint("LEFT", button, "LEFT", 40+2, 0)
		button.ItemIdSort:SetScript("OnClick", function() AllYouCanEat:GUIsort(2) end)

		button.ItemLinkSort = CreateFrame("Button", nil, button)
		TEMPLATE.SortPosition(button.ItemLinkSort, nil, "left", 19)
		button.ItemLinkSort:SetWidth(12)
		button.ItemLinkSort:SetHeight(12)
		button.ItemLinkSort:SetPoint("LEFT", button, "LEFT", 40+60+2+2, 0)
		button.ItemLinkSort:SetScript("OnClick", function() AllYouCanEat:GUIsort(3) end)

		button.BopItemSort = CreateFrame("Button", nil, button)
		TEMPLATE.SortPosition(button.BopItemSort, L["BoP"], "right", 35)
		button.BopItemSort:SetWidth(35)
		button.BopItemSort:SetHeight(12)
		button.BopItemSort:SetPoint("LEFT", button, "LEFT", 40+60+12+2+2+2, 0)
		button.BopItemSort:SetScript("OnClick", function() AllYouCanEat:GUIsort(4) end)

		button.QuestItemSort = CreateFrame("Button", nil, button)
		TEMPLATE.SortPosition(button.QuestItemSort, L["Q"], "right", 25)
		button.QuestItemSort:SetWidth(25)
		button.QuestItemSort:SetHeight(12)
		button.QuestItemSort:SetPoint("LEFT", button, "LEFT", 40+60+12+35+2+2+2+2, 0)
		button.QuestItemSort:SetScript("OnClick", function() AllYouCanEat:GUIsort(5) end)

		button.ItemNameSort = CreateFrame("Button", nil, button)
		TEMPLATE.SortPosition(button.ItemNameSort, L["Name"], "left", size.W_MainFrame-206) -- size.W_MainFrame-172-3-2-2-2-2-2-2-16-3
		button.ItemNameSort:SetWidth(size.W_MainFrame-206) -- size.W_MainFrame-172-3-2-2-2-2-2-2-16-3
		button.ItemNameSort:SetHeight(12)
		button.ItemNameSort:SetPoint("LEFT", button, "LEFT", 40+60+12+35+25+2+2+2+2+2, 0)
		button.ItemNameSort:SetScript("OnClick", function() AllYouCanEat:GUIsort(6) end)
	else
		button.NumberButton = button:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		button.NumberButton:SetWidth(40)
		button.NumberButton:SetHeight(12)
		button.NumberButton:SetPoint("LEFT", button, "LEFT", 0, 0)
		button.NumberButton:SetJustifyH("RIGHT")

		button.ItemIdButton = button:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		button.ItemIdButton:SetWidth(60)
		button.ItemIdButton:SetHeight(12)
		button.ItemIdButton:SetPoint("LEFT", button, "LEFT", 40+2, 0)
		button.ItemIdButton:SetTextColor(1, 1, 1, 1)
		button.ItemIdButton:SetJustifyH("RIGHT")

		button.ItemLinkButton = button:CreateTexture(nil, "ARTWORK")
		button.ItemLinkButton:SetWidth(12)--19
		button.ItemLinkButton:SetHeight(9/(19/12))--9
		button.ItemLinkButton:SetPoint("LEFT", button, "LEFT", 40+60+2+2, 0)
		button.ItemLinkButton:SetTexture("Interface\\TradeSkillFrame\\UI-TradeSkill-LinkButton")
		button.ItemLinkButton:SetAlpha(0.6)

		button.BopItemButton = button:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		button.BopItemButton:SetWidth(35)
		button.BopItemButton:SetHeight(12)
		button.BopItemButton:SetPoint("LEFT", button, "LEFT", 40+60+12+2+2+2, 0)
		button.BopItemButton:SetTextColor(1, 1, 1, 1)
		button.BopItemButton:SetJustifyH("RIGHT")

		button.QuestItemButton = button:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		button.QuestItemButton:SetWidth(25)
		button.QuestItemButton:SetHeight(12)
		button.QuestItemButton:SetPoint("LEFT", button, "LEFT", 40+60+12+35+2+2+2+2, 0)
		button.QuestItemButton:SetTextColor(1, 1, 1, 1)
		button.QuestItemButton:SetJustifyH("RIGHT")

		button.ItemNameButton = button:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		button.ItemNameButton:SetWidth(size.W_MainFrame-206) -- size.W_MainFrame-172-3-2-2-2-2-2-2-16-3
		button.ItemNameButton:SetHeight(12)
		button.ItemNameButton:SetPoint("LEFT", button, "LEFT", 40+60+12+35+25+2+2+2+2+2, 0)
		button.ItemNameButton:SetJustifyH("LEFT")

		button.Highlight = button:CreateTexture(nil, "ARTWORK")
		button.Highlight:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
		button.Highlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
		button.Highlight:SetTexture(1, 1, 1, 0.2)
		button:SetHighlightTexture(button.Highlight)
		
		button:SetScript("OnEnter", function() button.ItemLinkButton:SetAlpha(1) AllYouCanEat:ShowTooltip(button.itemID) end)
		button:SetScript("OnLeave", function() button.ItemLinkButton:SetAlpha(0.6) GameTooltip:Hide() end)
		button:SetScript("OnClick", function() AllYouCanEat:LinkWrapper(button.itemID) end)
	end
end
-- Template DataList END ----------------------------------------
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:InitOptions()
	SlashCmdList["ALLYOUCANEAT"] = function() AllYouCanEat:MainFrameToggle() end
	SLASH_ALLYOUCANEAT1 = "/allyoucaneat"
	SLASH_ALLYOUCANEAT2 = "/ayce"

	if AllYouCanEat_Options.version == nil then
		AllYouCanEat_Options.version = 3
	end

	if AllYouCanEat_Options.version < 3 then
		table.wipe(AllYouCanEat_Data)
		table.wipe(AllYouCanEat_Options)
		Print("Sorry. All AllYouCanEat data has been deleted because of a new storage system!")
		AllYouCanEat_Options.version = 3
	end

	if AllYouCanEat_Options.Data_SortBy    then GUI_SortBy         = AllYouCanEat_Options.Data_SortBy end
	if AllYouCanEat_Options.Data_SortOrder then GUI_SortOrder      = AllYouCanEat_Options.Data_SortOrder	end
	if AllYouCanEat_Options.Data_Detail    then GUI_Detail         = AllYouCanEat_Options.Data_Detail end
	if AllYouCanEat_Options.Data_Selection then GUI_Selection      = AllYouCanEat_Options.Data_Selection end

	if AllYouCanEat_Options.MinimapButton      == nil then AllYouCanEat_Options.MinimapButton      = false end
	if AllYouCanEat_Options.MinimapButtonPos   == nil then AllYouCanEat_Options.MinimapButtonPos   = -90   end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:LDBcheck()
	if LibStub and LibStub:GetLibrary("CallbackHandler-1.0", true) and LibStub:GetLibrary("LibDataBroker-1.1", true) then
		LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("AllYouCanEat", {
			type = "launcher",
			icon = AddonIcon,
			OnClick = function(self, button)
				AllYouCanEat:MainFrameToggle()
			end,
		})
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:CreateInterfaceOptions()
	GVAR.InterfaceOptions = CreateFrame("Frame", "AllYouCanEat_InterfaceOptions")
	GVAR.InterfaceOptions.name = "AllYouCanEat"

	GVAR.InterfaceOptions.Title = GVAR.InterfaceOptions:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	GVAR.InterfaceOptions.Title:SetPoint("TOPLEFT", 16, -16)
	GVAR.InterfaceOptions.Title:SetText("AllYouCanEat")
	GVAR.InterfaceOptions.Title:SetJustifyH("LEFT")
	GVAR.InterfaceOptions.Title:SetJustifyV("TOP")

	GVAR.InterfaceOptions.GUI = CreateFrame("Button", nil, GVAR.InterfaceOptions)
	TEMPLATE.TextButton(GVAR.InterfaceOptions.GUI, L["Open GUI"], 1)
	GVAR.InterfaceOptions.GUI:SetPoint("TOPLEFT", GVAR.InterfaceOptions.Title, "BOTTOMLEFT", 0, -10)
	GVAR.InterfaceOptions.GUI:SetWidth(150)
	GVAR.InterfaceOptions.GUI:SetHeight(22)
	GVAR.InterfaceOptions.GUI:SetScript("OnClick", function()
		InterfaceOptionsFrame_Show()
		HideUIPanel(GameMenuFrame)
		AllYouCanEat:MainFrameToggle()
	end)

	GVAR.InterfaceOptions.SlashCommandText = GVAR.InterfaceOptions:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.InterfaceOptions.SlashCommandText:SetPoint("LEFT", GVAR.InterfaceOptions.GUI, "RIGHT", 10, 0)
	GVAR.InterfaceOptions.SlashCommandText:SetText("/ayce")
	GVAR.InterfaceOptions.SlashCommandText:SetTextColor(1, 1, 0.49, 1)
	GVAR.InterfaceOptions.SlashCommandText:SetNonSpaceWrap(true)

	GVAR.InterfaceOptions.MinimapButton = CreateFrame("CheckButton", nil, GVAR.InterfaceOptions)
	TEMPLATE.CheckButton(GVAR.InterfaceOptions.MinimapButton, 16, 4, L["Minimap-Button"])
	GVAR.InterfaceOptions.MinimapButton:SetPoint("TOPLEFT", GVAR.InterfaceOptions.GUI, "BOTTOMLEFT", 0, -10)
	GVAR.InterfaceOptions.MinimapButton:SetChecked(AllYouCanEat_Options.MinimapButton)
	GVAR.InterfaceOptions.MinimapButton:SetScript("OnClick", function()
		AllYouCanEat_Options.MinimapButton = not AllYouCanEat_Options.MinimapButton
		AllYouCanEat:CreateMinimapButton()
	end)

	InterfaceOptions_AddCategory(GVAR.InterfaceOptions)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:SaveCharacterData()
	if not AllYouCanEat_Data.chars              then AllYouCanEat_Data.chars = {} end
	if not AllYouCanEat_Data.chars[playerRealm] then AllYouCanEat_Data.chars[playerRealm] = {}	end
	if not AllYouCanEat_Data.chars[playerRealm][playerGUID] then
		AllYouCanEat_Data.chars[playerRealm][playerGUID] = {}
		AllYouCanEat_Data.chars[playerRealm][playerGUID].name = playerName
		AllYouCanEat_Data.chars[playerRealm][playerGUID].class = playerClass
		AllYouCanEat_Data.chars[playerRealm][playerGUID].faction = playerFaction
		AllYouCanEat_Data.chars[playerRealm][playerGUID].savestats = false
		AllYouCanEat_Data.chars[playerRealm][playerGUID].tooltip = false
	else
		-- check if name, class or faction has changed
		if AllYouCanEat_Data.chars[playerRealm][playerGUID].name ~= playerName or
		   AllYouCanEat_Data.chars[playerRealm][playerGUID].class ~= playerClass or
		   AllYouCanEat_Data.chars[playerRealm][playerGUID].faction ~= playerFaction
		then
			AllYouCanEat_Data.chars[playerRealm][playerGUID].name = playerName
			AllYouCanEat_Data.chars[playerRealm][playerGUID].class = playerClass
			AllYouCanEat_Data.chars[playerRealm][playerGUID].faction = playerFaction
		end
	end
end

function AllYouCanEat:DeleteCharacterData(guid)
	AllYouCanEat_Data.chars[playerRealm][guid] = nil
end

function AllYouCanEat:SaveCharacterStatisticData()
	if not AllYouCanEat_Data.stats then AllYouCanEat_Data.stats = {} end

	for statID, data1 in pairs(DATA) do
		if not AllYouCanEat_Data.stats[statID] then AllYouCanEat_Data.stats[statID] = {} end
		for itemID, data2 in pairs(data1) do
			if itemID ~= "total" and itemID ~= "totalcompleted" then
				if not AllYouCanEat_Data.stats[statID][itemID] then AllYouCanEat_Data.stats[statID][itemID] = {} end
				if data2.completed then
					AllYouCanEat_Data.stats[statID][itemID][playerGUID] = 1
				else
					AllYouCanEat_Data.stats[statID][itemID][playerGUID] = 0
				end
			end
		end
	end

	AllYouCanEat_Data.chars[playerRealm][playerGUID].savestats = true
	AllYouCanEat_Data.chars[playerRealm][playerGUID].update = GUI_scantime.." | "..yearmonthday()
end

function AllYouCanEat:CheckSavedStatisticData()
	-- update saved character statistic data
	if AllYouCanEat_Data.chars[playerRealm][playerGUID].savestats then
		local savestats = AllYouCanEat_Data.chars[playerRealm][playerGUID].savestats
		local tooltip   = AllYouCanEat_Data.chars[playerRealm][playerGUID].tooltip
		AllYouCanEat:DeleteCharacterStatisticData(playerGUID)
		AllYouCanEat_Data.chars[playerRealm][playerGUID].savestats = savestats
		AllYouCanEat_Data.chars[playerRealm][playerGUID].tooltip   = tooltip
		AllYouCanEat:SaveCharacterStatisticData()
	end
end

function AllYouCanEat:DeleteCharacterStatisticData(guid)
	for statID, data1 in pairs(AllYouCanEat_Data.stats) do
		for itemID, data2 in pairs(data1) do
			if AllYouCanEat_Data.stats[statID][itemID][guid] then
				AllYouCanEat_Data.stats[statID][itemID][guid] = nil
			end
		end
	end

	AllYouCanEat_Data.chars[playerRealm][guid].savestats = false
	AllYouCanEat_Data.chars[playerRealm][guid].tooltip = false
	AllYouCanEat_Data.chars[playerRealm][guid].update = nil

	local x = 0
	for realm, guid in pairs(AllYouCanEat_Data.chars) do
		for key, value in pairs(guid) do
			if value.savestats == true then
				x = x + 1
				break
			end
		end
	end
	if x == 0 then
		AllYouCanEat_Data.stats = nil
	end
end

function AllYouCanEat:CheckIfTooltipCodeIsNeeded()
	local x = 0
	for playerguid, data in pairs(AllYouCanEat_Data.chars[playerRealm]) do
		if data.tooltip == true then
			x = x + 1
			break
		end
	end
	
	if x > 0 then
		TooltipCodeIsNeeded = true
	else
		TooltipCodeIsNeeded = false
	end
end

function AllYouCanEat:ConfirmPopup(message, what)
	if not StaticPopupDialogs["ALLYOUCANEAT_CONFIRM_DIALOG"] then
		StaticPopupDialogs["ALLYOUCANEAT_CONFIRM_DIALOG"] = {}
	end
	local t = StaticPopupDialogs["ALLYOUCANEAT_CONFIRM_DIALOG"]
	for k in pairs(t) do
		t[k] = nil
	end
	t.text = message
	t.button1 = ACCEPT or "Accept"
	t.button2 = CANCEL or "Cancel"
	t.OnAccept = function() AllYouCanEat:ChangeCharacterOption(what) end
	t.timeout = 0
	t.whileDead = 1
	t.hideOnEscape = 1
	StaticPopup_Show("ALLYOUCANEAT_CONFIRM_DIALOG")
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:TooltipInfoText(itemID, timeElapsed)
	GameTooltip:ClearLines()
	GameTooltip:AddLine("|TInterface\\DialogFrame\\UI-Dialog-Icon-AlertNew:24|t "..L["Unsafe Item"], 1, 0, 0)
	GameTooltip:AddLine(L["ItemID"]..": "..itemID)
	if timeElapsed then
		timeElapsed = math_floor((10 - timeElapsed) * mult2 + 0.5) / mult2
		GameTooltip:AddLine(string.format(L["You can try to start a new server query in %s seconds!"], "|cffffff7f"..timeElapsed.."|r"), 1, 1, 1, 1)
		GameTooltip:AddLine(L["You may be disconnected."], 1, 0.5, 0, 1)
	else
		GameTooltip:AddLine(L["Please wait 5 seconds for next server query."], 1, 1, 1, 1)
	end
end

function AllYouCanEat:ShowTooltip(itemID)
	local _, itemLink, itemRarity = GetItemInfo(itemID)

	GameTooltip:SetOwner(GVAR.DataFrame, "ANCHOR_BOTTOMRIGHT", 0, 20+(GUI_Data_curNum*12)) -- 3+(12)+2+(GUI_Data_curNum*12)+3 -- (382)

	if itemLink then
		ServerQueryTime[itemID] = nil
		GameTooltip:SetHyperlink(itemLink)
		GameTooltip:Show()
		if DATA[GUI_Selection][itemID].itemlink == 0 then
			DATA[GUI_Selection][itemID].itemlink = 1
			if itemRarity then
				DATA[GUI_Selection][itemID].itemrarity = itemRarity+1
			end

			local lineCount = GameTooltip:NumLines()
			local maxLine = 5
			local bop, quest
			if lineCount < 5 then
				maxLine = lineCount
			end
			for i = 1, maxLine do
				local text = _G["GameTooltipTextLeft"..i]:GetText()
				if text == ITEM_BIND_ON_PICKUP then
					bop = 1
				end
				if text == ITEM_BIND_QUEST then
					quest = 1
				end
			end

			if DATA[GUI_Selection][itemID].bopitem == 0 and bop then
				DATA[GUI_Selection][itemID].bopitem = 1
			end
			if DATA[GUI_Selection][itemID].questitem == 0 and quest then
				DATA[GUI_Selection][itemID].questitem = 1
			end

			if GUI_SortBy == 3 or GUI_SortBy == 4 or GUI_SortBy == 5 then
				AllYouCanEat:UpdateData()
			else
				for i = 1, GUI_Data_Count do
					if GUI_Data[i].itemid == itemID then
						GUI_Data[i].itemlink = 1
						if itemRarity then
							GUI_Data[i].itemrarity = itemRarity+1
						end
						if bop then
							GUI_Data[i].bopitem = 1
						end
						if quest then
							GUI_Data[i].questitem = 1
						end
						break
					end
				end
				AllYouCanEat:FillList()
			end
		end

	else
		if ServerQueryTime[itemID] then
			local timeElapsed = GetTime() - ServerQueryTime[itemID]
			if timeElapsed > 10 then
				ServerQueryTime[itemID] = nil
			else
				AllYouCanEat:TooltipInfoText(itemID, timeElapsed)
				GameTooltip:Show()
				return
			end
		else
			ServerQueryTime[itemID] = GetTime()
		end

		local QueryIsSave
		if GetTime() - ServerQueryTimeLast > 5 then
			QueryIsSave = true
		end

		if QueryIsSave then
			ServerQueryTimeLast = GetTime()
			GameTooltip:SetHyperlink("item:"..itemID)
			AllYouCanEat:TooltipInfoText(itemID, 0)
		else
			AllYouCanEat:TooltipInfoText(itemID)
		end
		GameTooltip:Show()
	end
end

function AllYouCanEat:AddNote(tooltip, item)
	if not TooltipCodeIsNeeded then return end

	local item = item or select(2, tooltip:GetItem())
	if not item then return end

	local itemID = select(3, item:find("item:(%d+)"))
	itemID = tonumber(itemID)
	if not itemID then return end

	local hasID
	for i = 1, #StatisticID do
		local statID = StatisticID[i].id
		if DATA[statID][itemID] then
			hasID = statID
			break
		end
	end
	if not hasID then return end

	local upval = true

	for guid, value in pairs(AllYouCanEat_Data.chars[playerRealm]) do
		if value.savestats and value.tooltip then

			local finish
			if AllYouCanEat_Data.stats and
			   AllYouCanEat_Data.stats[hasID] and
			   AllYouCanEat_Data.stats[hasID][itemID] and
			   AllYouCanEat_Data.stats[hasID][itemID][guid] and
			   AllYouCanEat_Data.stats[hasID][itemID][guid] == 1
			then
				finish = true
			end

			if upval then
				tooltip:AddLine("AllYouCanEat:", 1, 0.5, 0)
				upval = false
			end

			if finish then
				tooltip:AddDoubleLine("|cff"..ClassHexColor(value.class)..value.name.."|r", L["consumed"], 1, 1, 1,  0, 0.75, 0)
			else
				tooltip:AddDoubleLine("|cff"..ClassHexColor(value.class)..value.name.."|r", L["not consumed"], 1, 1, 1,  1, 0, 0)
			end

		end
	end

	if tooltip:IsShown() then tooltip:Show() end
end

function AllYouCanEat:LinkWrapper(itemID)
	local _, itemLink = GetItemInfo(itemID)
	if itemLink then
		if IsModifiedClick("CHATLINK") then
			if ChatEdit_InsertLink(itemLink) then
				return true
			end
		end
		if ChatEdit_InsertLink(itemLink) then
			return true
		end
		return false
	end
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
local function SetFrameHeight(height)
	local hx = size.H_SelectionAllNormal

	local hmf = height + size.H_Headline-1 + hx-1 + size.H_Status-1 -- height + 28-1 + 88-1 + 20-1
	GVAR.MainFrame:SetHeight(hmf)
	GVAR.MainFrame:SetClampRectInsets(0, 0, 0, hmf-size.H_Headline)

	GVAR.DataFrame:SetHeight(height)

	local hcm = height + hx-1 -- height + 28-1 + 88-1
	GVAR.DataManager:SetHeight(hcm)

	if (hcm-3-3) <= size.H_charmanagerScrollFrame then
		GVAR.DataManager.ScrollFrame.ScrollBarScrollUpButton:SetAlpha(1)
		GVAR.DataManager.ScrollFrame.ScrollBarScrollDownButton:SetAlpha(1)
		GVAR.DataManager.ScrollFrame.ScrollBarThumbTexture:SetAlpha(1)
	else
		GVAR.DataManager.ScrollFrame.ScrollBarScrollUpButton:SetAlpha(0.2)
		GVAR.DataManager.ScrollFrame.ScrollBarScrollDownButton:SetAlpha(0.2)
		GVAR.DataManager.ScrollFrame.ScrollBarThumbTexture:SetAlpha(0.2)
		GVAR.DataManager.ScrollFrame.ScrollBarScrollUpButton:Disable()
		GVAR.DataManager.ScrollFrame.ScrollBarScrollDownButton:Disable()
	end
end

local function SetFrameWidth(width)
	GVAR.MainFrame:SetWidth(width)
	GVAR.HeadlineFrame:SetWidth(width)
	GVAR.SelectionFrame:SetWidth(width)
	GVAR.SelectionFrame.StatisticButton[StatisticID[1].id]:SetPoint("TOPLEFT", GVAR.SelectionFrame, "TOPLEFT", (width - ((#StatisticID*32)+((#StatisticID-1)*size.W_SelectionButtonSpace))) / 2, -28)

	local w1 = ( width-(3*112)-(2*3) ) / 2
	GVAR.SelectionFrame.ButtonAll:SetPoint("BOTTOMLEFT", GVAR.SelectionFrame, "BOTTOMLEFT", w1, -1)
	GVAR.SelectionFrame.ButtonComplete:SetPoint("BOTTOMLEFT", GVAR.SelectionFrame, "BOTTOMLEFT", w1+115, -1) -- , w1+112+3, -1)
	GVAR.SelectionFrame.ButtonIncomplete:SetPoint("BOTTOMLEFT", GVAR.SelectionFrame, "BOTTOMLEFT", w1+230, -1) -- , w1+112+3+112+3, -1)

	GVAR.DataFrame:SetWidth(width)
	GVAR.DataFrame.Sort:SetWidth(width)
	for i = 1, GUI_Data_maxNum do
		GVAR.DataFrame.ScrollFrameButton[i]:SetWidth(width-24) -- width-3-3-2-16
		GVAR.DataFrame.ScrollFrameButton[i].ItemNameButton:SetWidth(width-206) -- width-172-3-2-2-2-2-2-2-16-3
	end
	GVAR.DataFrame.Sort.ItemNameSort:SetWidth(width-206) -- width-172-3-2-2-2-2-2-2-16-3
	GVAR.StatusFrame:SetWidth(width)
	GVAR.DataManager:SetWidth(width)
	GVAR.DataManager.RealmName:SetPoint("TOPLEFT", GVAR.DataManager.ScrollFrame.ScrollChildFrame, "TOPLEFT", (width-24-320)/2, -10)
end

local function CalculateFrameSize(width, height)
	local height_data = 20+(GUI_Data_curNum*12) -- 3+(1*12)+2+(GUI_Data_curNum*12)+3
	
	local hx = size.H_SelectionAllNormal
	
	local height = height - size.H_Headline+1 - hx+1 - size.H_Status+1
	if height < height_data then
		local diff = height_data - height
		local freq = math_floor(diff/12)
		if freq > 0 then
			GUI_Data_curNum = GUI_Data_curNum - freq
			AllYouCanEat:FillList()
			height = 20+(GUI_Data_curNum*12) -- 3+(1*12)+2+(GUI_Data_curNum*12)+3
		end
	else
		local diff = height - height_data
		local freq = math_floor(diff/12)
		if freq > 0 then
			GUI_Data_curNum = GUI_Data_curNum + freq
			AllYouCanEat:FillList()
			height = 20+(GUI_Data_curNum*12) -- 3+(1*12)+2+(GUI_Data_curNum*12)+3
		end
	end
	
	SetFrameHeight(height)
	SetFrameWidth(width)
end

local function ResizeCheck(frame)
	local width = math_floor(frame:GetWidth() + 0.5)
	local height = math_floor(frame:GetHeight() + 0.5)
	CalculateFrameSize(width, height)
end

local function ResizeEnd(frame)
	frame:SetScript("OnUpdate", nil)
	frame:StopMovingOrSizing()

	SetFrameHeight( 20+(GUI_Data_curNum*12) ) -- 3+(1*12)+2+(GUI_Data_curNum*12)+3
	SetFrameWidth( math_floor(frame:GetWidth() + 0.5) )

	AllYouCanEat:SaveMainFramePos()
end

local function ResizeStart(frame, point)
	frame:SetScript("OnUpdate", function() ResizeCheck(frame) end)
	frame:StartSizing(point)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:CreateMainFrame()
	GVAR.MainFrame = CreateFrame("Frame", "AllYouCanEat_MainFrame", UIParent)
	GVAR.MainFrame:EnableMouse(true)
	GVAR.MainFrame:SetMovable(true)
	GVAR.MainFrame:SetResizable(true)
	GVAR.MainFrame:SetMinResize(size.W_MinMainFrame, size.H_MainframeMinNormal) -- (minWidth, minHeight)
	GVAR.MainFrame:SetMaxResize(size.W_MaxMainFrame, size.H_MainframeMaxNormal) -- (maxWidth, maxHeight)
	GVAR.MainFrame:SetToplevel(true)
	GVAR.MainFrame:SetClampedToScreen(true)
	GVAR.MainFrame:SetClampRectInsets(0, 0, 0, size.H_MainframeNormal-size.H_Headline)
	GVAR.MainFrame:Hide()
	GVAR.MainFrame:SetWidth(size.W_MainFrame)
	GVAR.MainFrame:SetHeight(size.H_MainframeNormal)
	GVAR.MainFrame:SetScript("OnShow", function() AllYouCanEat:MainFrameShow() end)
	GVAR.MainFrame:SetScript("OnHide", function() AllYouCanEat:MainFrameHide() end)



	-- Headline Frame START
	GVAR.HeadlineFrame = CreateFrame("Frame", nil, GVAR.MainFrame)
	TEMPLATE.BorderTRBL(GVAR.HeadlineFrame)
	GVAR.HeadlineFrame:SetWidth(size.W_MainFrame)
	GVAR.HeadlineFrame:SetHeight(size.H_Headline)
	GVAR.HeadlineFrame:SetPoint("TOPLEFT", GVAR.MainFrame, "TOPLEFT", 0, 0)
	GVAR.HeadlineFrame:EnableMouse(true)

	GVAR.HeadlineFrame.TitleTexture = GVAR.HeadlineFrame:CreateTexture(nil, "ARTWORK")
	GVAR.HeadlineFrame.TitleTexture:SetWidth(20)
	GVAR.HeadlineFrame.TitleTexture:SetHeight(20)
	GVAR.HeadlineFrame.TitleTexture:SetPoint("LEFT", GVAR.HeadlineFrame, "LEFT", 4, 0)
	GVAR.HeadlineFrame.TitleTexture:SetTexture(AddonIcon)

	GVAR.HeadlineFrame.Title = GVAR.HeadlineFrame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	GVAR.HeadlineFrame.Title:SetPoint("LEFT", GVAR.HeadlineFrame.TitleTexture, "RIGHT", 4, 0)
	GVAR.HeadlineFrame.Title:SetJustifyH("RIGHT")
	GVAR.HeadlineFrame.Title:SetText("AllYouCanEat")

	GVAR.HeadlineFrame.Close = CreateFrame("Button", nil, GVAR.HeadlineFrame)
	TEMPLATE.IconButton(GVAR.HeadlineFrame.Close, 1)
	GVAR.HeadlineFrame.Close:SetWidth(20)
	GVAR.HeadlineFrame.Close:SetHeight(20)
	GVAR.HeadlineFrame.Close:SetPoint("RIGHT", GVAR.HeadlineFrame, "RIGHT", -4, 0)
	GVAR.HeadlineFrame.Close:SetScript("OnClick", function() AllYouCanEat:MainFrameToggle() end)

	GVAR.HeadlineFrame.Options = CreateFrame("Button", nil, GVAR.HeadlineFrame)
	TEMPLATE.IconButton(GVAR.HeadlineFrame.Options, 2)
	GVAR.HeadlineFrame.Options:SetWidth(20)
	GVAR.HeadlineFrame.Options:SetHeight(20)
	GVAR.HeadlineFrame.Options:SetPoint("RIGHT", GVAR.HeadlineFrame.Close, "LEFT", -4, 0)
	GVAR.HeadlineFrame.Options:SetScript("OnClick", function() AllYouCanEat:MainFrameToggle() InterfaceOptionsFrame_OpenToCategory("AllYouCanEat") end)

	GVAR.HeadlineFrame.DataManager = CreateFrame("Button", nil, GVAR.HeadlineFrame)
	TEMPLATE.TextButton(GVAR.HeadlineFrame.DataManager, L["Data Manager"], 1)
	GVAR.HeadlineFrame.DataManager:SetWidth( GVAR.HeadlineFrame.DataManager:GetTextWidth() + 40 )
	GVAR.HeadlineFrame.DataManager:SetHeight(20)
	GVAR.HeadlineFrame.DataManager:SetPoint("RIGHT", GVAR.HeadlineFrame.Options, "LEFT", -4, 0)
	GVAR.HeadlineFrame.DataManager:SetScript("OnClick", function() AllYouCanEat:DataManagerToggle() end)

	GVAR.HeadlineFrame.Mover = CreateFrame("Button", nil, GVAR.HeadlineFrame)
	GVAR.HeadlineFrame.Mover:SetWidth(22)
	GVAR.HeadlineFrame.Mover:SetHeight(size.H_Headline)
	GVAR.HeadlineFrame.Mover:SetPoint("LEFT", GVAR.HeadlineFrame, "LEFT", 0, 0)
	GVAR.HeadlineFrame.Mover:SetPoint("RIGHT", GVAR.HeadlineFrame.DataManager, "LEFT", 0, 0)
	GVAR.HeadlineFrame.MoverTexture = GVAR.HeadlineFrame.Mover:CreateTexture(nil, "BACKGROUND")
	GVAR.HeadlineFrame.MoverTexture:SetWidth(1)
	GVAR.HeadlineFrame.MoverTexture:SetHeight(1)
	GVAR.HeadlineFrame.MoverTexture:SetPoint("LEFT", GVAR.HeadlineFrame.Title, "RIGHT", 0, 0)
	GVAR.HeadlineFrame.MoverTexture:SetPoint("TOPRIGHT", GVAR.HeadlineFrame.Mover, "TOPRIGHT", 0, 0)
	GVAR.HeadlineFrame.MoverTexture:SetPoint("BOTTOMRIGHT", GVAR.HeadlineFrame.Mover, "BOTTOMRIGHT", 0, 0)
	GVAR.HeadlineFrame.MoverTexture:SetTexture(1, 1, 1, 0)
	GVAR.HeadlineFrame.Mover:SetScript("OnEnter", function() AllYouCanEat:MoverInfo("MOVE", "TOP") end)
	GVAR.HeadlineFrame.Mover:SetScript("OnLeave", function() AllYouCanEat:MoverInfo() end)
	GVAR.HeadlineFrame.Mover:SetScript("OnMouseDown", function() GVAR.MainFrame:StartMoving() end)
	GVAR.HeadlineFrame.Mover:SetScript("OnMouseUp", function() GVAR.MainFrame:StopMovingOrSizing() AllYouCanEat:SaveMainFramePos() end)
	-- Headline Frame END



	-- Selection Frame START
	GVAR.SelectionFrame = CreateFrame("Frame", nil, GVAR.MainFrame)
	TEMPLATE.BorderTRBL(GVAR.SelectionFrame)
	GVAR.SelectionFrame:SetWidth(size.W_MainFrame)
	GVAR.SelectionFrame:SetHeight(size.H_SelectionAllNormal)
	GVAR.SelectionFrame:SetPoint("TOPLEFT", GVAR.HeadlineFrame, "BOTTOMLEFT", 0, 1)
	GVAR.SelectionFrame:EnableMouse(true)

	local function OnLeave(self)
		if not self:IsMouseOver() and not GVAR.SelectionFrame.CurrentChar:IsMouseOver() and not GVAR.SelectionFrame.CharList:IsMouseOver() then
			GVAR.SelectionFrame.CharList:Hide()
			GVAR.SelectionFrame.CurrentChar.Normal:SetTexture("Interface\\Buttons\\UI-Panel-ExpandButton-Up")
		end
	end

	GVAR.SelectionFrame.CurrentChar = CreateFrame("Button", nil, GVAR.SelectionFrame)
	TEMPLATE.DropDownBox(GVAR.SelectionFrame.CurrentChar)
	TEMPLATE.BorderTRBL(GVAR.SelectionFrame.CurrentChar)
	GVAR.SelectionFrame.CurrentChar:SetWidth(140)
	GVAR.SelectionFrame.CurrentChar:SetHeight(20)
	GVAR.SelectionFrame.CurrentChar:SetPoint("TOP", GVAR.SelectionFrame, "TOP", -72.5, -4) -- 155 = 150 search box +5 space / 2
	GVAR.SelectionFrame.CurrentChar:SetScript("OnClick", function() AllYouCanEat:ToggleCharList() end)
	GVAR.SelectionFrame.CurrentChar:SetScript("OnLeave", function(self) OnLeave(self) end)

	GVAR.SelectionFrame.CharList = CreateFrame("Frame", nil, GVAR.SelectionFrame)
	TEMPLATE.BorderTRBL(GVAR.SelectionFrame.CharList)
	GVAR.SelectionFrame.CharList:SetToplevel(true)
	GVAR.SelectionFrame.CharList:SetWidth(140)
	GVAR.SelectionFrame.CharList:SetHeight(GUI_CharListData_maxNum*12+8)
	GVAR.SelectionFrame.CharList:SetPoint("TOPLEFT", GVAR.SelectionFrame.CurrentChar, "BOTTOMLEFT", 0, 1)
	GVAR.SelectionFrame.CharList:SetScript("OnLeave", function(self) OnLeave(self) end)
	GVAR.SelectionFrame.CharList:Hide()

	for i = 1, GUI_CharListData_maxNum do
		if not GVAR.SelectionFrame.CharListButton then GVAR.SelectionFrame.CharListButton = {} end
		GVAR.SelectionFrame.CharListButton[i] = CreateFrame("Button", nil, GVAR.SelectionFrame.CharList)
		GVAR.SelectionFrame.CharListButton[i]:SetWidth(140)
		GVAR.SelectionFrame.CharListButton[i]:SetHeight(12)
		if i == 1 then
			GVAR.SelectionFrame.CharListButton[i]:SetPoint("TOPLEFT", GVAR.SelectionFrame.CharList, "TOPLEFT", 4, -4)
		else
			GVAR.SelectionFrame.CharListButton[i]:SetPoint("TOPLEFT", GVAR.SelectionFrame.CharListButton[(i-1)], "BOTTOMLEFT", 0, 0)
		end

		GVAR.SelectionFrame.CharListButton[i].Name = GVAR.SelectionFrame.CharListButton[i]:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		GVAR.SelectionFrame.CharListButton[i].Name:SetWidth(140)
		GVAR.SelectionFrame.CharListButton[i].Name:SetHeight(12)
		GVAR.SelectionFrame.CharListButton[i].Name:SetPoint("LEFT", GVAR.SelectionFrame.CharListButton[i], "LEFT", 0, 0)
		GVAR.SelectionFrame.CharListButton[i].Name:SetTextColor(1, 1, 1, 1)
		GVAR.SelectionFrame.CharListButton[i].Name:SetJustifyH("LEFT")

		GVAR.SelectionFrame.CharListButton[i].Background = GVAR.SelectionFrame.CharListButton[i]:CreateTexture(nil, "ARTWORK")
		GVAR.SelectionFrame.CharListButton[i].Background:SetPoint("TOPLEFT", GVAR.SelectionFrame.CharListButton[i], "TOPLEFT", 0, 0)
		GVAR.SelectionFrame.CharListButton[i].Background:SetPoint("BOTTOMRIGHT", GVAR.SelectionFrame.CharListButton[i], "BOTTOMRIGHT", -8, 0)
		GVAR.SelectionFrame.CharListButton[i].Background:SetTexture(1, 1, 1, 0)

		GVAR.SelectionFrame.CharListButton[i].Highlight = GVAR.SelectionFrame.CharListButton[i]:CreateTexture(nil, "ARTWORK")
		GVAR.SelectionFrame.CharListButton[i].Highlight:SetPoint("TOPLEFT", GVAR.SelectionFrame.CharListButton[i], "TOPLEFT", 0, 0)
		GVAR.SelectionFrame.CharListButton[i].Highlight:SetPoint("BOTTOMRIGHT", GVAR.SelectionFrame.CharListButton[i], "BOTTOMRIGHT", -8, 0)
		GVAR.SelectionFrame.CharListButton[i].Highlight:SetTexture(1, 1, 1, 0.2)
		GVAR.SelectionFrame.CharListButton[i]:SetHighlightTexture(GVAR.SelectionFrame.CharListButton[i].Highlight)
		
		GVAR.SelectionFrame.CharListButton[i]:SetScript("OnClick", function(self) AllYouCanEat:SelectNewMainFrameData(self.guid) end)
		GVAR.SelectionFrame.CharListButton[i]:SetScript("OnLeave", function(self) OnLeave(self) end)
	end
	AllYouCanEat:UpdateCharSelection()

	GVAR.SelectionFrame.SearchBox = CreateFrame("Button", nil, GVAR.SelectionFrame)
	TEMPLATE.EditBox(GVAR.SelectionFrame.SearchBox, 150, true, AllYouCanEat.Search, AllYouCanEat.SearchClear)
	GVAR.SelectionFrame.SearchBox:SetWidth(150)
	GVAR.SelectionFrame.SearchBox:SetHeight(17)
	GVAR.SelectionFrame.SearchBox:SetPoint("LEFT", GVAR.SelectionFrame.CurrentChar, "RIGHT", 5, 0)

	for i = 1, #StatisticID do
		if not GVAR.SelectionFrame.StatisticButton then GVAR.SelectionFrame.StatisticButton = {} end
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id] = CreateFrame("Button", nil, GVAR.SelectionFrame)
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id]:SetWidth(32)
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id]:SetHeight(32)
		if i == 1 then
			GVAR.SelectionFrame.StatisticButton[StatisticID[i].id]:SetPoint("TOPLEFT", GVAR.SelectionFrame, "TOPLEFT", (size.W_MainFrame - ((#StatisticID*32)+((#StatisticID-1)*size.W_SelectionButtonSpace))) / 2, -28)
		else
			GVAR.SelectionFrame.StatisticButton[StatisticID[i].id]:SetPoint("LEFT", GVAR.SelectionFrame.StatisticButton[StatisticID[i-1].id], "RIGHT", size.W_SelectionButtonSpace, 0)
		end
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].Icon = GVAR.SelectionFrame.StatisticButton[StatisticID[i].id]:CreateTexture(nil, "BACKGROUND")
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].Icon:SetWidth(32)
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].Icon:SetHeight(32)
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].Icon:SetAllPoints(GVAR.SelectionFrame.StatisticButton[StatisticID[i].id])
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].Icon:SetTexture(StatisticID[i].icon)
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].IconBorder = GVAR.SelectionFrame.StatisticButton[StatisticID[i].id]:CreateTexture(nil, "ARTWORK")
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].IconBorder:SetWidth(64)
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].IconBorder:SetHeight(64)
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].IconBorder:SetPoint("TOPLEFT", -14, 14)
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].IconBorder:SetPoint("BOTTOMRIGHT", 14, -14)
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].IconBorder:SetTexture("Interface\\Buttons\\CheckButtonGlow")
		
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id]:SetScript("OnClick", function()
			if GUI_Selection == StatisticID[i].id then return end
			AllYouCanEat_Options.Data_Selection = StatisticID[i].id
			GUI_Selection = StatisticID[i].id
			AllYouCanEat:UpdateSelectionButtons()
			AllYouCanEat:UpdateData()
		end)

		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id]:SetScript("OnEnter", function()
			GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].Icon:SetAlpha(1)
		end)

		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id]:SetScript("OnLeave", function()
			if GUI_Selection == StatisticID[i].id then return end
			GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].Icon:SetAlpha(0.7)
		end)

		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].TextCompleted = GVAR.SelectionFrame.StatisticButton[StatisticID[i].id]:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].TextCompleted:SetPoint("TOP", GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].Icon, "BOTTOM", 0, 0)
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].TextCompleted:SetHeight(12)
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].TextCompleted:SetJustifyH("CENTER")
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].TextCompleted:SetTextColor(1, 1, 1, 1)
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].TextTotal = GVAR.SelectionFrame.StatisticButton[StatisticID[i].id]:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].TextTotal:SetPoint("TOP", GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].TextCompleted, "BOTTOM", 0, 0)
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].TextTotal:SetHeight(12)
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].TextTotal:SetJustifyH("CENTER")
		GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].TextTotal:SetTextColor(0.6, 0.6, 0.6, 1)
	end

	AllYouCanEat:UpdateSelectionButtons()

	local w1 = ( size.W_MainFrame-(3*112)-(2*3) ) / 2

	GVAR.SelectionFrame.ButtonAll = CreateFrame("Button", nil, GVAR.SelectionFrame)
	TEMPLATE.TabButton(GVAR.SelectionFrame.ButtonAll, L["all"], 1)
	GVAR.SelectionFrame.ButtonAll:SetWidth(112)
	GVAR.SelectionFrame.ButtonAll:SetHeight(20)
	GVAR.SelectionFrame.ButtonAll:SetPoint("BOTTOMLEFT", GVAR.SelectionFrame, "BOTTOMLEFT", w1, -1)
	GVAR.SelectionFrame.ButtonAll:SetScript("OnClick", function()
		if GUI_Detail == 1 then return end
		AllYouCanEat_Options.Data_Detail = 1
		GUI_Detail = 1
		AllYouCanEat:UpdateData()
	end)

	GVAR.SelectionFrame.ButtonComplete = CreateFrame("Button", nil, GVAR.SelectionFrame)
	TEMPLATE.TabButton(GVAR.SelectionFrame.ButtonComplete, L["consumed"], 2)
	GVAR.SelectionFrame.ButtonComplete:SetWidth(112)
	GVAR.SelectionFrame.ButtonComplete:SetHeight(20)
	GVAR.SelectionFrame.ButtonComplete:SetPoint("BOTTOMLEFT", GVAR.SelectionFrame, "BOTTOMLEFT", w1+115, -1)
	GVAR.SelectionFrame.ButtonComplete:SetScript("OnClick", function()
		if GUI_Detail == 2 then return end
		AllYouCanEat_Options.Data_Detail = 2
		GUI_Detail = 2
		AllYouCanEat:UpdateData()
	end)

	GVAR.SelectionFrame.ButtonIncomplete = CreateFrame("Button", nil, GVAR.SelectionFrame)
	TEMPLATE.TabButton(GVAR.SelectionFrame.ButtonIncomplete, L["not consumed"], 3)
	GVAR.SelectionFrame.ButtonIncomplete:SetWidth(112)
	GVAR.SelectionFrame.ButtonIncomplete:SetHeight(20)
	GVAR.SelectionFrame.ButtonIncomplete:SetPoint("BOTTOMLEFT", GVAR.SelectionFrame, "BOTTOMLEFT", w1+230, -1)
	GVAR.SelectionFrame.ButtonIncomplete:SetScript("OnClick", function()
		if GUI_Detail == 3 then return end
		AllYouCanEat_Options.Data_Detail = 3
		GUI_Detail = 3
		AllYouCanEat:UpdateData()
	end)
	-- Selection Frame END



	-- Data Frame START
	GVAR.DataFrame = CreateFrame("Frame", nil, GVAR.MainFrame)
	TEMPLATE.BorderTRBL(GVAR.DataFrame)
	GVAR.DataFrame:SetWidth(size.W_MainFrame)
	GVAR.DataFrame:SetHeight( 20+(GUI_Data_curNum*12) ) -- 3+(12)+2+(GUI_Data_curNum*12)+3
	GVAR.DataFrame:SetPoint("TOPLEFT", GVAR.SelectionFrame, "BOTTOMLEFT", 0, 1)
	GVAR.DataFrame:EnableMouse(true)
	GVAR.DataFrame:EnableMouseWheel(true)
	GVAR.DataFrame:SetScript("OnMouseWheel", NOOP)

	GVAR.DataFrame.Sort = CreateFrame("Frame", nil, GVAR.DataFrame)
	TEMPLATE.DataList(GVAR.DataFrame.Sort, true)
	GVAR.DataFrame.Sort:SetWidth(size.W_MainFrame)
	GVAR.DataFrame.Sort:SetHeight(12)
	GVAR.DataFrame.Sort:SetPoint("TOPLEFT", GVAR.DataFrame, "TOPLEFT", 3, -3)

	GVAR.DataFrame.ScrollFrame = CreateFrame("ScrollFrame", nil, GVAR.DataFrame)
	TEMPLATE.ScrollFrame(GVAR.DataFrame.ScrollFrame)
	GVAR.DataFrame.ScrollFrame:SetPoint("TOPLEFT", GVAR.DataFrame, "TOPLEFT", 3, -17) -- 3, -(12+2+3)
	GVAR.DataFrame.ScrollFrame:SetPoint("BOTTOMRIGHT", GVAR.DataFrame, "BOTTOMRIGHT", -21, 3) -- -(16+3+2), 3
	GVAR.DataFrame.ScrollFrame:SetScript("OnVerticalScroll", function(self, value) AllYouCanEat:OnVerticalScroll(value) end)

	for i = 1, GUI_Data_maxNum do
		if not GVAR.DataFrame.ScrollFrameButton then GVAR.DataFrame.ScrollFrameButton = {} end
		GVAR.DataFrame.ScrollFrameButton[i] = CreateFrame("Button", nil, GVAR.DataFrame)
		TEMPLATE.DataList(GVAR.DataFrame.ScrollFrameButton[i])
		GVAR.DataFrame.ScrollFrameButton[i]:SetWidth(size.W_MainFrame-3-3-16-2)
		GVAR.DataFrame.ScrollFrameButton[i]:SetHeight(12)
		if i == 1 then
			GVAR.DataFrame.ScrollFrameButton[i]:SetPoint("TOPLEFT", GVAR.DataFrame, "TOPLEFT", 3, -17) -- 3, -(12+2+3)
		else
			GVAR.DataFrame.ScrollFrameButton[i]:SetPoint("TOPLEFT", GVAR.DataFrame.ScrollFrameButton[i-1], "BOTTOMLEFT", 0, 0)
		end
	end
	-- Data Frame END



	-- Status Frame START
	GVAR.StatusFrame = CreateFrame("Frame", nil, GVAR.MainFrame)
	TEMPLATE.BorderTRBL(GVAR.StatusFrame)
	GVAR.StatusFrame:SetWidth(size.W_MainFrame)
	GVAR.StatusFrame:SetHeight(size.H_Status)
	GVAR.StatusFrame:SetPoint("TOPLEFT", GVAR.DataFrame, "BOTTOMLEFT", 0, 1)
	GVAR.StatusFrame:EnableMouse(true)
	GVAR.StatusFrame:SetScript("OnEnter", function() AllYouCanEat:MoverInfo("MOVE", "BOTTOM") end)
	GVAR.StatusFrame:SetScript("OnLeave", function() AllYouCanEat:MoverInfo() end)
	GVAR.StatusFrame:SetScript("OnMouseDown", function() GVAR.MainFrame:StartMoving() end)
	GVAR.StatusFrame:SetScript("OnMouseUp", function() GVAR.MainFrame:StopMovingOrSizing() AllYouCanEat:SaveMainFramePos() end)

	GVAR.StatusFrameText = GVAR.StatusFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.StatusFrameText:SetHeight(20)
	GVAR.StatusFrameText:SetPoint("CENTER", GVAR.StatusFrame)
	GVAR.StatusFrameText:SetTextColor(1, 1, 1, 1)
	GVAR.StatusFrameText:SetJustifyH("CENTER")

	GVAR.StatusFrame.ResizeBL = CreateFrame("Button", nil, GVAR.StatusFrame)
	GVAR.StatusFrame.ResizeBL:SetPoint("BOTTOMLEFT", GVAR.StatusFrame, "BOTTOMLEFT", 0, 0)
	GVAR.StatusFrame.ResizeBL:SetWidth(20)
	GVAR.StatusFrame.ResizeBL:SetHeight(20)

	GVAR.StatusFrame.ResizeBLtexture = GVAR.StatusFrame.ResizeBL:CreateTexture(nil, "ARTWORK")
	GVAR.StatusFrame.ResizeBLtexture:SetPoint("TOPLEFT", 2, -2)
	GVAR.StatusFrame.ResizeBLtexture:SetPoint("BOTTOMRIGHT", -2, 2)
	GVAR.StatusFrame.ResizeBLtexture:SetAlpha(0.6)
	GVAR.StatusFrame.ResizeBLtexture:SetTexture("Interface\\Minimap\\Rotating-MinimapGroupArrow")
	GVAR.StatusFrame.ResizeBLtexture:SetTexCoord(0.78125,0.484375 , 0.515625,0.21875 , 0.515625,0.75 , 0.25,0.484375) -- magic

	GVAR.StatusFrame.ResizeBL:SetScript("OnMouseDown", function()
		GVAR.StatusFrame.ResizeBLtexture:SetPoint("TOPLEFT", 3, -3)
		GVAR.StatusFrame.ResizeBLtexture:SetPoint("BOTTOMRIGHT", -3, 3)
		ResizeStart(GVAR.MainFrame, "BOTTOMLEFT")
	end)
	GVAR.StatusFrame.ResizeBL:SetScript("OnMouseUp", function()
		GVAR.StatusFrame.ResizeBLtexture:SetPoint("TOPLEFT", 2, -2)
		GVAR.StatusFrame.ResizeBLtexture:SetPoint("BOTTOMRIGHT", -2, 2)
		ResizeEnd(GVAR.MainFrame)
	end)
	GVAR.StatusFrame.ResizeBL:SetScript("OnEnter", function()
		AllYouCanEat:MoverInfo("RESIZE", "BOTTOM")
		GVAR.StatusFrame.ResizeBLtexture:SetAlpha(1)
	end)
	GVAR.StatusFrame.ResizeBL:SetScript("OnLeave", function()
		AllYouCanEat:MoverInfo()
		GVAR.StatusFrame.ResizeBLtexture:SetAlpha(0.6)
	end)

	GVAR.StatusFrame.ResizeBR = CreateFrame("Button", nil, GVAR.StatusFrame)
	GVAR.StatusFrame.ResizeBR:SetPoint("BOTTOMRIGHT", GVAR.StatusFrame, "BOTTOMRIGHT", 0, 0)
	GVAR.StatusFrame.ResizeBR:SetWidth(20)
	GVAR.StatusFrame.ResizeBR:SetHeight(20)

	GVAR.StatusFrame.ResizeBRtexture = GVAR.StatusFrame.ResizeBR:CreateTexture(nil, "ARTWORK")
	GVAR.StatusFrame.ResizeBRtexture:SetPoint("TOPLEFT", 2, -2)
	GVAR.StatusFrame.ResizeBRtexture:SetPoint("BOTTOMRIGHT", -2, 2)
	GVAR.StatusFrame.ResizeBRtexture:SetAlpha(0.6)
	GVAR.StatusFrame.ResizeBRtexture:SetTexture("Interface\\Minimap\\Rotating-MinimapGroupArrow")
	GVAR.StatusFrame.ResizeBRtexture:SetTexCoord(0.515625,0.75 , 0.78125,0.484375 , 0.25,0.484375 , 0.515625,0.21875) -- magic

	GVAR.StatusFrame.ResizeBR:SetScript("OnMouseDown", function()
		GVAR.StatusFrame.ResizeBRtexture:SetPoint("TOPLEFT", 3, -3)
		GVAR.StatusFrame.ResizeBRtexture:SetPoint("BOTTOMRIGHT", -3, 3)
		ResizeStart(GVAR.MainFrame, "BOTTOMRIGHT")
	end)
	GVAR.StatusFrame.ResizeBR:SetScript("OnMouseUp", function()
		GVAR.StatusFrame.ResizeBRtexture:SetPoint("TOPLEFT", 2, -2)
		GVAR.StatusFrame.ResizeBRtexture:SetPoint("BOTTOMRIGHT", -2, 2)
		ResizeEnd(GVAR.MainFrame)
	end)
	GVAR.StatusFrame.ResizeBR:SetScript("OnEnter", function()
		AllYouCanEat:MoverInfo("RESIZE", "BOTTOM")
		GVAR.StatusFrame.ResizeBRtexture:SetAlpha(1)
	end)
	GVAR.StatusFrame.ResizeBR:SetScript("OnLeave", function()
		AllYouCanEat:MoverInfo()
		GVAR.StatusFrame.ResizeBRtexture:SetAlpha(0.6)
	end)
	-- Status Frame END



	-- MoverInfo Frame START
	GVAR.MoverInfoTopFrame = CreateFrame("Frame", nil, GVAR.MainFrame)
	GVAR.MoverInfoTopFrame:SetWidth(150)
	GVAR.MoverInfoTopFrame:SetHeight(14)
	GVAR.MoverInfoTopFrame:SetPoint("BOTTOM", GVAR.MainFrame, "TOP", 0, 0)
	GVAR.MoverInfoTopFrame:EnableMouse(true)
	GVAR.MoverInfoTopFrame:Hide()
	GVAR.MoverInfoTopFrameTexture = GVAR.MoverInfoTopFrame:CreateTexture(nil, "BACKGROUND")
	GVAR.MoverInfoTopFrameTexture:SetAllPoints(GVAR.MoverInfoTopFrame)
	GVAR.MoverInfoTopFrameTexture:SetTexture(0, 0, 0, 0.5)
	GVAR.MoverInfoTopFrameText = GVAR.MoverInfoTopFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.MoverInfoTopFrameText:SetPoint("CENTER", GVAR.MoverInfoTopFrame, "CENTER", 0, 0)
	GVAR.MoverInfoTopFrameText:SetJustifyH("CENTER")
	GVAR.MoverInfoTopFrameText:SetTextColor(0.8, 0.8, 0.8, 1)

	GVAR.MoverInfoBottomFrame = CreateFrame("Frame", nil, GVAR.MainFrame)
	GVAR.MoverInfoBottomFrame:SetWidth(150)
	GVAR.MoverInfoBottomFrame:SetHeight(14)
	GVAR.MoverInfoBottomFrame:SetPoint("TOP", GVAR.StatusFrame, "BOTTOM", 0, 0)
	GVAR.MoverInfoBottomFrame:EnableMouse(true)
	GVAR.MoverInfoBottomFrame:Hide()
	GVAR.MoverInfoBottomFrameTexture = GVAR.MoverInfoBottomFrame:CreateTexture(nil, "BACKGROUND")
	GVAR.MoverInfoBottomFrameTexture:SetAllPoints(GVAR.MoverInfoBottomFrame)
	GVAR.MoverInfoBottomFrameTexture:SetTexture(0, 0, 0, 0.5)
	GVAR.MoverInfoBottomFrameText = GVAR.MoverInfoBottomFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.MoverInfoBottomFrameText:SetPoint("CENTER", GVAR.MoverInfoBottomFrame, "CENTER", 0, 0)
	GVAR.MoverInfoBottomFrameText:SetJustifyH("CENTER")
	GVAR.MoverInfoBottomFrameText:SetTextColor(0.8, 0.8, 0.8, 1)
	-- MoverInfo Frame END



	-- DataManager START
	GVAR.DataManager = CreateFrame("Frame", nil, GVAR.MainFrame)
	TEMPLATE.BorderTRBL(GVAR.DataManager)
	GVAR.DataManager:SetWidth(size.W_MainFrame)
	GVAR.DataManager:SetHeight(size.H_SelectionAllNormal + (20+(GUI_Data_curNum*12)) - 1) -- 3+(12)+2+(GUI_Data_curNum*12)+3
	GVAR.DataManager:SetPoint("TOPLEFT", GVAR.HeadlineFrame, "BOTTOMLEFT", 0, 1)
	GVAR.DataManager:EnableMouse(true)
	GVAR.DataManager:EnableMouseWheel(true)
	GVAR.DataManager:SetScript("OnMouseWheel", NOOP)
	GVAR.DataManager:Hide()

	GVAR.DataManager.ScrollFrame = CreateFrame("ScrollFrame", nil, GVAR.DataManager)
	TEMPLATE.ScrollFrame(GVAR.DataManager.ScrollFrame, true)
	GVAR.DataManager.ScrollFrame:SetPoint("TOPLEFT", GVAR.DataManager, "TOPLEFT", 3, -3) -- 3, -(12+2+3)
	GVAR.DataManager.ScrollFrame:SetPoint("BOTTOMRIGHT", GVAR.DataManager, "BOTTOMRIGHT", -21, 3) -- -(16+3+2), 3
	GVAR.DataManager.ScrollFrame:SetScript("OnVerticalScroll", function(self, value)
		GVAR.DataManager.ScrollFrame.ScrollBar:SetValue(value)
		GVAR.DataManager.ScrollFrame.offset = math_floor((value / 12) + 0.5)
		GVAR.DataManager.ScrollFrame.ScrollChildFrame:SetPoint("TOPLEFT", 0, value)

		local a1 = math_floor((size.H_charmanagerScrollFrame-value) + 0.5) + 16
		local a2 = math_floor(GVAR.DataManager:GetHeight() + 0.5)

		if value == 0 then
			GVAR.DataManager.ScrollFrame.ScrollBarScrollUpButton:Disable()
		else
			GVAR.DataManager.ScrollFrame.ScrollBarScrollUpButton:Enable()
		end
		if a1 == a2 then
			GVAR.DataManager.ScrollFrame.ScrollBarScrollDownButton:Disable()
		else
			GVAR.DataManager.ScrollFrame.ScrollBarScrollDownButton:Enable()
		end
	end)
	GVAR.DataManager.ScrollFrame:SetScrollChild(GVAR.DataManager.ScrollFrame.ScrollChildFrame)

	GVAR.DataManager.RealmName = GVAR.DataManager.ScrollFrame.ScrollChildFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.DataManager.RealmName:SetWidth(320)
	GVAR.DataManager.RealmName:SetHeight(12)
	GVAR.DataManager.RealmName:SetPoint("TOPLEFT", GVAR.DataManager.ScrollFrame.ScrollChildFrame, "TOPLEFT", (size.W_MainFrame-24-320)/2, -10)
	GVAR.DataManager.RealmName:SetJustifyH("LEFT")

	GVAR.DataManager.CharList = CreateFrame("Frame", nil, GVAR.DataManager.ScrollFrame.ScrollChildFrame)
	TEMPLATE.BorderTRBL(GVAR.DataManager.CharList)
	GVAR.DataManager.CharList:SetWidth(320)
	GVAR.DataManager.CharList:SetHeight(GUI_CharListData_maxNum*12+8)
	GVAR.DataManager.CharList:SetPoint("TOPLEFT", GVAR.DataManager.RealmName, "BOTTOMLEFT", 0, -10)

	for i = 1, GUI_CharListData_maxNum do
		if not GVAR.DataManager.CharListButton then GVAR.DataManager.CharListButton = {} end
		GVAR.DataManager.CharListButton[i] = CreateFrame("Button", nil, GVAR.DataManager.CharList)
		GVAR.DataManager.CharListButton[i]:SetWidth(320)
		GVAR.DataManager.CharListButton[i]:SetHeight(12)
		if i == 1 then
			GVAR.DataManager.CharListButton[i]:SetPoint("TOPLEFT", GVAR.DataManager.CharList, "TOPLEFT", 4, -4)
		else
			GVAR.DataManager.CharListButton[i]:SetPoint("TOPLEFT", GVAR.DataManager.CharListButton[(i-1)], "BOTTOMLEFT", 0, 0)
		end

		GVAR.DataManager.CharListButton[i].Name = GVAR.DataManager.CharListButton[i]:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		GVAR.DataManager.CharListButton[i].Name:SetWidth(100)
		GVAR.DataManager.CharListButton[i].Name:SetHeight(12)
		GVAR.DataManager.CharListButton[i].Name:SetPoint("LEFT", GVAR.DataManager.CharListButton[i], "LEFT", 0, 0)
		GVAR.DataManager.CharListButton[i].Name:SetTextColor(1, 1, 1, 1)
		GVAR.DataManager.CharListButton[i].Name:SetJustifyH("LEFT")

		GVAR.DataManager.CharListButton[i].SaveStats = GVAR.DataManager.CharListButton[i]:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		GVAR.DataManager.CharListButton[i].SaveStats:SetWidth(110)
		GVAR.DataManager.CharListButton[i].SaveStats:SetHeight(12)
		GVAR.DataManager.CharListButton[i].SaveStats:SetPoint("LEFT", GVAR.DataManager.CharListButton[i], "LEFT", 100, 0)
		GVAR.DataManager.CharListButton[i].SaveStats:SetTextColor(1, 1, 1, 1)
		GVAR.DataManager.CharListButton[i].SaveStats:SetJustifyH("CENTER")

		GVAR.DataManager.CharListButton[i].Tooltip = GVAR.DataManager.CharListButton[i]:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		GVAR.DataManager.CharListButton[i].Tooltip:SetWidth(110-8)
		GVAR.DataManager.CharListButton[i].Tooltip:SetHeight(12)
		GVAR.DataManager.CharListButton[i].Tooltip:SetPoint("LEFT", GVAR.DataManager.CharListButton[i], "LEFT", 210, 0)
		GVAR.DataManager.CharListButton[i].Tooltip:SetTextColor(1, 1, 1, 1)
		GVAR.DataManager.CharListButton[i].Tooltip:SetJustifyH("CENTER")

		GVAR.DataManager.CharListButton[i].Background = GVAR.DataManager.CharListButton[i]:CreateTexture(nil, "ARTWORK")
		GVAR.DataManager.CharListButton[i].Background:SetPoint("TOPLEFT", GVAR.DataManager.CharListButton[i], "TOPLEFT", 0, 0)
		GVAR.DataManager.CharListButton[i].Background:SetPoint("BOTTOMRIGHT", GVAR.DataManager.CharListButton[i], "BOTTOMRIGHT", -8, 0)
		GVAR.DataManager.CharListButton[i].Background:SetTexture(1, 1, 1, 0)

		GVAR.DataManager.CharListButton[i].CurChar = GVAR.DataManager.CharListButton[i]:CreateTexture(nil, "ARTWORK")
		GVAR.DataManager.CharListButton[i].CurChar:SetWidth(12)
		GVAR.DataManager.CharListButton[i].CurChar:SetHeight(12)
		GVAR.DataManager.CharListButton[i].CurChar:SetPoint("RIGHT", GVAR.DataManager.CharListButton[i], "LEFT", -10, 0)
		GVAR.DataManager.CharListButton[i].CurChar:SetTexture("Interface\\PetBattles\\PetBattle-StatIcons")
		GVAR.DataManager.CharListButton[i].CurChar:SetTexCoord(0, 0.5, 0.5, 1) -- 0/32, 16/32, 16/32, 32/32

		GVAR.DataManager.CharListButton[i].Highlight = GVAR.DataManager.CharListButton[i]:CreateTexture(nil, "ARTWORK")
		GVAR.DataManager.CharListButton[i].Highlight:SetPoint("TOPLEFT", GVAR.DataManager.CharListButton[i], "TOPLEFT", 0, 0)
		GVAR.DataManager.CharListButton[i].Highlight:SetPoint("BOTTOMRIGHT", GVAR.DataManager.CharListButton[i], "BOTTOMRIGHT", -8, 0)
		GVAR.DataManager.CharListButton[i].Highlight:SetTexture(1, 1, 1, 0.2)
		GVAR.DataManager.CharListButton[i]:SetHighlightTexture(GVAR.DataManager.CharListButton[i].Highlight)

		GVAR.DataManager.CharListButton[i]:SetScript("OnClick", function(self) AllYouCanEat:SetupCharacterOptions(self.guid) end)
	end

	GVAR.DataManager.CharName = GVAR.DataManager.ScrollFrame.ScrollChildFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.DataManager.CharName:SetWidth(370)
	GVAR.DataManager.CharName:SetHeight(12)
	GVAR.DataManager.CharName:SetPoint("TOPLEFT", GVAR.DataManager.CharList, "BOTTOMLEFT", 0, -20)
	GVAR.DataManager.CharName:SetJustifyH("LEFT")

	GVAR.DataManager.Opt_SaveStatusText = GVAR.DataManager.ScrollFrame.ScrollChildFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.DataManager.Opt_SaveStatusText:SetPoint("TOPLEFT", GVAR.DataManager.CharName, "BOTTOMLEFT", 0, -10)
	GVAR.DataManager.Opt_SaveStatusText:SetTextColor(1, 1, 1, 1)
	GVAR.DataManager.Opt_SaveStatusText:SetNonSpaceWrap(true)
	GVAR.DataManager.Opt_SaveStatusText:SetJustifyH("LEFT")

	GVAR.DataManager.Opt_SaveStats = CreateFrame("Button", nil, GVAR.DataManager.ScrollFrame.ScrollChildFrame)
	TEMPLATE.TextButton(GVAR.DataManager.Opt_SaveStats, L["Save Data"], 1)
	GVAR.DataManager.Opt_SaveStats:SetWidth(155)
	GVAR.DataManager.Opt_SaveStats:SetHeight(24)
	GVAR.DataManager.Opt_SaveStats:SetPoint("TOPLEFT", GVAR.DataManager.Opt_SaveStatusText, "BOTTOMLEFT", 0, -5)
	GVAR.DataManager.Opt_SaveStats:SetScript("OnClick", function()
		AllYouCanEat:ChangeCharacterOption("savestats")
	end)
	TEMPLATE.EnableTextButton(GVAR.DataManager.Opt_SaveStats, 1)

	GVAR.DataManager.Opt_DeleteStats = CreateFrame("Button", nil, GVAR.DataManager.ScrollFrame.ScrollChildFrame)
	TEMPLATE.TextButton(GVAR.DataManager.Opt_DeleteStats, L["Delete Data"], 1)
	GVAR.DataManager.Opt_DeleteStats:SetWidth(155)
	GVAR.DataManager.Opt_DeleteStats:SetHeight(24)
	GVAR.DataManager.Opt_DeleteStats:SetPoint("TOPLEFT", GVAR.DataManager.Opt_SaveStats, "TOPRIGHT", 10, 0)
	GVAR.DataManager.Opt_DeleteStats:SetScript("OnClick", function()
		AllYouCanEat:ConfirmPopup(L["Do you really want to delete the data from this character?"], "deletestats")
	end)
	TEMPLATE.EnableTextButton(GVAR.DataManager.Opt_DeleteStats, 1)

	GVAR.DataManager.Opt_SaveStats_Text = GVAR.DataManager.ScrollFrame.ScrollChildFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.DataManager.Opt_SaveStats_Text:SetPoint("TOPLEFT", GVAR.DataManager.Opt_SaveStats, "BOTTOMLEFT", 0, -10)
	GVAR.DataManager.Opt_SaveStats_Text:SetTextColor(1, 1, 1, 1)
	GVAR.DataManager.Opt_SaveStats_Text:SetNonSpaceWrap(true)
	GVAR.DataManager.Opt_SaveStats_Text:SetWidth(320)
	GVAR.DataManager.Opt_SaveStats_Text:SetJustifyH("LEFT")
	GVAR.DataManager.Opt_SaveStats_Text:SetText(L["Saves the statistics data from this character to SavedVariable, so you can view it with other characters."])

	GVAR.DataManager.Opt_ShowInTooltip = CreateFrame("CheckButton", nil, GVAR.DataManager.ScrollFrame.ScrollChildFrame)
	TEMPLATE.CheckButton(GVAR.DataManager.Opt_ShowInTooltip, 14, 4, L["Show statistic data in GameTooltip"])
	GVAR.DataManager.Opt_ShowInTooltip:SetPoint("TOPLEFT", GVAR.DataManager.Opt_SaveStats_Text, "BOTTOMLEFT", 0, -20)
	GVAR.DataManager.Opt_ShowInTooltip:SetScript("OnClick", function()
		AllYouCanEat:ChangeCharacterOption("tooltip")
	end)

	GVAR.DataManager.Opt_RemoveChar = CreateFrame("Button", nil, GVAR.DataManager.ScrollFrame.ScrollChildFrame)
	TEMPLATE.TextButton(GVAR.DataManager.Opt_RemoveChar, L["Remove Character"], 1)
	GVAR.DataManager.Opt_RemoveChar:SetWidth(180)
	GVAR.DataManager.Opt_RemoveChar:SetHeight(24)
	GVAR.DataManager.Opt_RemoveChar:SetPoint("TOPLEFT", GVAR.DataManager.Opt_ShowInTooltip, "BOTTOMLEFT", 0, -20)
	GVAR.DataManager.Opt_RemoveChar:SetScript("OnClick", function()
		AllYouCanEat:ConfirmPopup(L["Do you really want to remove this character?"], "removechar")
	end)
	TEMPLATE.EnableTextButton(GVAR.DataManager.Opt_RemoveChar, 1)

	GVAR.DataManager.Opt_RemoveChar_Text = GVAR.DataManager.ScrollFrame.ScrollChildFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.DataManager.Opt_RemoveChar_Text:SetPoint("TOPLEFT", GVAR.DataManager.Opt_RemoveChar, "BOTTOMLEFT", 0, -5)
	GVAR.DataManager.Opt_RemoveChar_Text:SetTextColor(1, 1, 1, 1)
	GVAR.DataManager.Opt_RemoveChar_Text:SetNonSpaceWrap(true)
	GVAR.DataManager.Opt_RemoveChar_Text:SetWidth(320)
	GVAR.DataManager.Opt_RemoveChar_Text:SetJustifyH("LEFT")
	GVAR.DataManager.Opt_RemoveChar_Text:SetText("("..L["Only useful if a character was permanently deleted."]..")")
	-- DataManager END

	--local h = 10 + 12 + 10 + (GUI_CharListData_maxNum*12+8) + 20 + 12 + 10 + 12 + 5 + 24 + 5 + GVAR.DataManager.Opt_SaveStats_Text:GetStringHeight() + 20 + 14 + 20 + 24 + 5 + GVAR.DataManager.Opt_RemoveChar_Text:GetHeight()
	local h = 10 + 12 + 10 + (GUI_CharListData_maxNum*12+8) + 20 + 12 + 10 + 12 + 5 + 24 + 5 +  30  + 20 + 14 + 20 + 24 + 5 +  20

	size.H_charmanagerScrollFrame = math_floor(h + 0.5)

	GVAR.DataManager.ScrollFrame:SetHeight(size.H_charmanagerScrollFrame)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:CreateMinimapButton()
	if not AllYouCanEat_Options.MinimapButton then
		if AllYouCanEat_MinimapButton then
			AllYouCanEat_MinimapButton:Hide()
		end
		return
	else
		if AllYouCanEat_MinimapButton then
			AllYouCanEat_MinimapButton:Show()
			return
		end
	end

	if AllYouCanEat_MinimapButton then return end

	local function MoveMinimapButton()
		local xpos
		local ypos
		local minimapShape = GetMinimapShape and GetMinimapShape() or "ROUND"
		if minimapShape == "SQUARE" then
			xpos = 110 * cos(AllYouCanEat_Options.MinimapButtonPos or 0)
			ypos = 110 * sin(AllYouCanEat_Options.MinimapButtonPos or 0)
			xpos = math.max(-82, math.min(xpos, 84))
			ypos = math.max(-86, math.min(ypos, 82))
		else
			xpos = 80 * cos(AllYouCanEat_Options.MinimapButtonPos or 0)
			ypos = 80 * sin(AllYouCanEat_Options.MinimapButtonPos or 0)
		end
		AllYouCanEat_MinimapButton:SetPoint("TOPLEFT", Minimap, "TOPLEFT", 54-xpos, ypos-54)
	end

	local function DragMinimapButton()
		local xpos, ypos = GetCursorPosition()
		local xmin, ymin = Minimap:GetLeft() or 400, Minimap:GetBottom() or 400
		local scale = Minimap:GetEffectiveScale()
		xpos = xmin-xpos/scale+70
		ypos = ypos/scale-ymin-70
		AllYouCanEat_Options.MinimapButtonPos = math.deg(math.atan2(ypos, xpos))
		MoveMinimapButton()
	end

	local MinimapButton = CreateFrame("Button", "AllYouCanEat_MinimapButton", Minimap)
	MinimapButton:EnableMouse(true)
	MinimapButton:SetMovable(true)
	MinimapButton:SetToplevel(true)
	MinimapButton:SetWidth(32)
	MinimapButton:SetHeight(32)
	MinimapButton:SetPoint("TOPLEFT")
	MinimapButton:SetFrameStrata("LOW")
	MinimapButton:RegisterForClicks("AnyUp")
	MinimapButton:RegisterForDrag("LeftButton")

	local texture = MinimapButton:CreateTexture(nil, "ARTWORK")
	texture:SetWidth(54)
	texture:SetHeight(54)
	texture:SetPoint("TOPLEFT")
	texture:SetTexture("Interface\\Minimap\\MiniMap-TrackingBorder")

	local texture = MinimapButton:CreateTexture(nil, "BACKGROUND")
	texture:SetWidth(24)
	texture:SetHeight(24)
	texture:SetPoint("TOPLEFT", 2, -4)
	texture:SetTexture("Interface\\Minimap\\UI-Minimap-Background")

	local NormalTexture = MinimapButton:CreateTexture(nil, "ARTWORK")
	NormalTexture:SetWidth(12)
	NormalTexture:SetHeight(14)
	NormalTexture:SetPoint("TOPLEFT", 10, -8)
	NormalTexture:SetTexture(AddonIcon)
	NormalTexture:SetTexCoord(2/16, 14/16, 1/16, 15/16)
	MinimapButton:SetNormalTexture(NormalTexture)

	local PushedTexture = MinimapButton:CreateTexture(nil, "ARTWORK")
	PushedTexture:SetWidth(10)
	PushedTexture:SetHeight(12)
	PushedTexture:SetPoint("TOPLEFT", 11, -9)
	PushedTexture:SetTexture(AddonIcon)
	PushedTexture:SetTexCoord(2/16, 14/16, 1/16, 15/16)
	MinimapButton:SetPushedTexture(PushedTexture)

	local HighlightTexture = MinimapButton:CreateTexture(nil, "ARTWORK")
	HighlightTexture:SetPoint("TOPLEFT", 0, 0)
	HighlightTexture:SetPoint("BOTTOMRIGHT", 0, 0)
	HighlightTexture:SetTexture("Interface\\Minimap\\UI-Minimap-ZoomButton-Highlight")
	MinimapButton:SetHighlightTexture(HighlightTexture)

	MinimapButton:SetScript("OnEnter", function(self)
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:AddLine("AllYouCanEat", 1, 0.82, 0, 1)
		GameTooltip:Show()
	end)
	MinimapButton:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
	MinimapButton:SetScript("OnClick", function(self, button) AllYouCanEat:MainFrameToggle() end)
	MinimapButton:SetScript("OnDragStart", function(self) self:LockHighlight() self:SetScript("OnUpdate", DragMinimapButton) end)
	MinimapButton:SetScript("OnDragStop", function(self) self:SetScript("OnUpdate", nil) self:UnlockHighlight() end)

	MoveMinimapButton()
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:ChangeCharacterOption(what)
	local guid = GVAR.DataManager.CharName.guid

	if what == "savestats" then
		AllYouCanEat:SaveCharacterStatisticData()
	elseif what == "deletestats" then
		AllYouCanEat:DeleteCharacterStatisticData(guid)
		guid = playerGUID
		AllYouCanEat:SelectNewMainFrameData(guid)
	elseif what == "removechar" then
		AllYouCanEat:DeleteCharacterStatisticData(guid)
		AllYouCanEat:DeleteCharacterData(guid)
		
		for i = 1, GUI_CharListData_maxNum do
			GVAR.SelectionFrame.CharListButton[i]:Hide()
			GVAR.DataManager.CharListButton[i]:Hide()
		end
		local x = 0
		for k in pairs(AllYouCanEat_Data.chars[playerRealm]) do
			x = x + 1
		end
		GUI_CharListData_maxNum = x
		AllYouCanEat:UpdateCharSelection()
		
		guid = playerGUID
	elseif what == "tooltip" then
		for playerrealm in pairs(AllYouCanEat_Data.chars) do
			for playerguid, data in pairs(AllYouCanEat_Data.chars[playerrealm]) do
				if playerguid == guid then
					data.tooltip = not data.tooltip
					break
				end
			end
		end
		AllYouCanEat:CheckIfTooltipCodeIsNeeded()
	end
	AllYouCanEat:SetupCharacterOptions(guid)
end

function AllYouCanEat:SetupCharacterOptions(guid)
	local name = ""
	local class = "PRIEST"
	local tooltip = false
	local savestats = false

	for playerrealm in pairs(AllYouCanEat_Data.chars) do
		for playerguid, data in pairs(AllYouCanEat_Data.chars[playerrealm]) do
			if playerguid == guid then
				name = data.name
				class = data.class
				if data.tooltip then
					tooltip = true
				end
				if data.savestats then
					savestats = true
				end
				break
			end
		end
	end

	-- realmname START
	GVAR.DataManager.RealmName:SetText(playerRealm)
	-- realmname END

	-- charname START
	GVAR.DataManager.CharName:SetText(L["Options for:"].." |cff"..ClassHexColor(class)..name.."|r".." |cffffffff("..guid..")|r")
	GVAR.DataManager.CharName.guid = guid
	-- charname END

	-- save status START
	if savestats then
		GVAR.DataManager.Opt_SaveStatusText:SetText(L["Status:"].." "..L["SAVED"])
		GVAR.DataManager.Opt_SaveStatusText:SetTextColor(0, 0.75, 0, 1)
	else
		GVAR.DataManager.Opt_SaveStatusText:SetText(L["Status:"].." "..L["NOT SAVED"])
		GVAR.DataManager.Opt_SaveStatusText:SetTextColor(1, 0, 0, 1)
	end
	-- save status END

	-- save/delete button START
	if playerGUID == guid then
		if savestats then
			TEMPLATE.DisableTextButton(GVAR.DataManager.Opt_SaveStats)
			TEMPLATE.EnableTextButton(GVAR.DataManager.Opt_DeleteStats, 1)
		else
			TEMPLATE.EnableTextButton(GVAR.DataManager.Opt_SaveStats, 1)
			TEMPLATE.DisableTextButton(GVAR.DataManager.Opt_DeleteStats)
		end
		GVAR.DataManager.Opt_SaveStats_Text:SetTextColor(1, 1, 1, 1)
	else
		if savestats then
			TEMPLATE.DisableTextButton(GVAR.DataManager.Opt_SaveStats)
			TEMPLATE.EnableTextButton(GVAR.DataManager.Opt_DeleteStats, 1)
		else
			TEMPLATE.DisableTextButton(GVAR.DataManager.Opt_SaveStats)
			TEMPLATE.DisableTextButton(GVAR.DataManager.Opt_DeleteStats)
		end
		GVAR.DataManager.Opt_SaveStats_Text:SetTextColor(0.3, 0.3, 0.3, 1)
	end
	-- save/delete button END

	-- tooltip START
	if savestats then
		TEMPLATE.EnableCheckButton(GVAR.DataManager.Opt_ShowInTooltip)
	else
		TEMPLATE.DisableCheckButton(GVAR.DataManager.Opt_ShowInTooltip)
	end
	GVAR.DataManager.Opt_ShowInTooltip:SetChecked(tooltip)
	-- tooltip END
	
	-- remove char START
	if playerGUID == guid then
		TEMPLATE.DisableTextButton(GVAR.DataManager.Opt_RemoveChar)
		GVAR.DataManager.Opt_RemoveChar_Text:SetTextColor(0.3, 0.3, 0.3, 1)
	else
		TEMPLATE.EnableTextButton(GVAR.DataManager.Opt_RemoveChar, 1)
		GVAR.DataManager.Opt_RemoveChar_Text:SetTextColor(1, 1, 1, 1)
	end
	-- remove char END
	AllYouCanEat:UpdateData_CharList()
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:UpdateData_CharList()
	table.wipe(GUI_CharListData)

	local x = 1
	for k, v in pairs(AllYouCanEat_Data.chars[playerRealm]) do
		GUI_CharListData[x] = {}
		GUI_CharListData[x].name = v.name
		GUI_CharListData[x].class = v.class
		if v.savestats then
			GUI_CharListData[x].savestats = "|cff00bf00"..L["SAVED"].."|r"
		else
			GUI_CharListData[x].savestats = "|cffff0000"..L["NOT SAVED"].."|r"
		end
		if v.tooltip then
			GUI_CharListData[x].tooltip = L["TOOLTIP"]
		end
		GUI_CharListData[x].guid = k
		x = x + 1
	end

	table.sort(GUI_CharListData, function(a, b)
		if a.name < b.name then
			return true
		end
	end)

	for i = 1, GUI_CharListData_maxNum do
		GVAR.DataManager.CharListButton[i]:Hide()
	end

	for i = 1, #GUI_CharListData do
		if GUI_CharListData[i].guid == GVAR.DataManager.CharName.guid then
			GVAR.DataManager.CharListButton[i].Background:SetTexture(1, 1, 1, 0.1)
		else
			GVAR.DataManager.CharListButton[i].Background:SetTexture(1, 1, 1, 0)
		end

		if GUI_CharListData[i].guid == playerGUID then
			GVAR.DataManager.CharListButton[i].CurChar:Show()
		else
			GVAR.DataManager.CharListButton[i].CurChar:Hide()
		end

		GVAR.DataManager.CharListButton[i].Name:SetText("|cff"..ClassHexColor(GUI_CharListData[i].class)..GUI_CharListData[i].name.."|r")
		GVAR.DataManager.CharListButton[i].SaveStats:SetText(GUI_CharListData[i].savestats)
		GVAR.DataManager.CharListButton[i].Tooltip:SetText(GUI_CharListData[i].tooltip)
		GVAR.DataManager.CharListButton[i]:Show()
		GVAR.DataManager.CharListButton[i].guid = GUI_CharListData[i].guid
		GVAR.DataManager.CharListButton[i]:SetScript("OnClick", function(self) AllYouCanEat:SetupCharacterOptions(self.guid) end)
	end
	GVAR.DataManager.CharList:SetHeight(#GUI_CharListData*12+8)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:ToggleCharList(force)
	if GVAR.SelectionFrame.CharList:IsShown() or force then
		GVAR.SelectionFrame.CharList:Hide()
		GVAR.SelectionFrame.CurrentChar.Normal:SetTexture("Interface\\Buttons\\UI-Panel-ExpandButton-Up")
	else
		GVAR.SelectionFrame.CharList:Show()
		GVAR.SelectionFrame.CurrentChar.Normal:SetTexture("Interface\\Buttons\\UI-Panel-CollapseButton-Up")
	end
end

function AllYouCanEat:UpdateCharSelection()
	local temp = {}

	local x = 1
	for k, v in pairs(AllYouCanEat_Data.chars[playerRealm]) do
		temp[x] = {}
		temp[x].name = v.name
		temp[x].class = v.class
		temp[x].savestats = v.savestats
		temp[x].guid = k
		x = x + 1
	end

	table.sort(temp, function(a, b)
		if a.name < b.name then
			return true
		end
	end)

	for i = 1, GUI_CharListData_maxNum do
		GVAR.SelectionFrame.CharListButton[i]:Hide()
	end

	if GUI_CharListData_maxNum > myMAX_CHARACTERS_PER_REALM then
		Print(string.format(L["AllYouCanEat has saved more than %d characters for this realm (maximum: %d)."], myMAX_CHARACTERS_PER_REALM, myMAX_CHARACTERS_PER_REALM))
		Print(L["Please remove the permanently deleted character(s):"])
		Print(L["AllYouCanEat GUI -> 'DataManager' -> select char -> 'Remove Character'"])
	end

	local currName
	for i = 1, #temp do
		if temp[i].guid == GUI_MainDataGUID then
			GVAR.SelectionFrame.CharListButton[i].Background:SetTexture(1, 1, 1, 0.1)
			currName = "|cff"..ClassHexColor(temp[i].class)..temp[i].name.."|r"
		else
			GVAR.SelectionFrame.CharListButton[i].Background:SetTexture(1, 1, 1, 0)
		end

		GVAR.SelectionFrame.CharListButton[i].Name:SetText("|cff"..ClassHexColor(temp[i].class)..temp[i].name.."|r")
		GVAR.SelectionFrame.CharListButton[i]:Show()
		GVAR.SelectionFrame.CharListButton[i].guid = temp[i].guid
		GVAR.SelectionFrame.CharListButton[i]:SetScript("OnClick", function(self) AllYouCanEat:SelectNewMainFrameData(self.guid) end)
		if not temp[i].savestats and temp[i].guid ~= playerGUID then
			GVAR.SelectionFrame.CharListButton[i]:SetScript("OnClick", nil)
			GVAR.SelectionFrame.CharListButton[i].Name:SetText("|cff444444"..temp[i].name.."|r")
		end
	end

	GVAR.SelectionFrame.CurrentChar.Text:SetText(currName)
	GVAR.SelectionFrame.CharList:SetHeight(#temp*12+8)

	table.wipe(temp)
end

function AllYouCanEat:SelectNewMainFrameData(guid)
	GUI_MainDataGUID = guid
	AllYouCanEat:UpdateCharSelection()
	AllYouCanEat:ToggleCharList(1)
	AllYouCanEat:UpdateData()
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:InitializeFrame()
	AllYouCanEat:UpdateSortButtons()
	AllYouCanEat:UpdateData()
end

function AllYouCanEat:MainFrameToggle()
	if GVAR.MainFrame:IsShown() then
		GVAR.MainFrame:Hide()
	else
		GVAR.DataFrame:Show()
		GVAR.SelectionFrame:Show()
		if AllYouCanEat_Options.MainFrame_width and AllYouCanEat_Options.MainFrame_height then
			CalculateFrameSize(AllYouCanEat_Options.MainFrame_width, AllYouCanEat_Options.MainFrame_height)
		else
			CalculateFrameSize(size.W_MainFrame, size.H_MainframeNormal)
		end
		GVAR.MainFrame:Show()
	end
end

function AllYouCanEat:DataManagerToggle(force)
	if GVAR.DataManager:IsShown() or force then
		GVAR.DataManager:Hide()
		GVAR.HeadlineFrame.DataManager:SetText(L["Data Manager"])
		GVAR.SelectionFrame:Show()
		GVAR.DataFrame:Show()
		AllYouCanEat:UpdateStatusText() -- status text update
		GVAR.StatusFrame:SetPoint("TOPLEFT", GVAR.DataFrame, "BOTTOMLEFT", 0, 1)
	else
		GVAR.DataManager:Show()
		AllYouCanEat:SetupCharacterOptions(playerGUID)
		GVAR.HeadlineFrame.DataManager:SetText(L["Main Frame"])
		GVAR.SelectionFrame:Hide()
		GVAR.DataFrame:Hide()
		GVAR.StatusFrameText:SetText("") -- status text update
		GVAR.StatusFrame:SetPoint("TOPLEFT", GVAR.DataManager, "BOTTOMLEFT", 0, 1)
	end
end

function AllYouCanEat:MainFramePos()
	if AllYouCanEat_Options.MainFrame_relP then
		GVAR.MainFrame:ClearAllPoints()
		GVAR.MainFrame:SetPoint(AllYouCanEat_Options.MainFrame_relP, AllYouCanEat_Options.MainFrame_posX, AllYouCanEat_Options.MainFrame_posY)
	else
		GVAR.MainFrame:ClearAllPoints()
		GVAR.MainFrame:SetPoint("CENTER", 0, 100)
	end
end

function AllYouCanEat:MainFrameHide()
	PlaySound("igQuestListClose")
	GVAR.InterfaceOptions.GUI:Enable()
	AllYouCanEat:DataManagerToggle(1)
end

function AllYouCanEat:MainFrameShow()
	PlaySound("igQuestListOpen")
	GVAR.InterfaceOptions.GUI:Disable()
	AllYouCanEat:MainFramePos()
	AllYouCanEat:UpdateData()
end

function AllYouCanEat:SaveMainFramePos()
	local point, relativeTo, relativePoint, x, y = GVAR.MainFrame:GetPoint()
	AllYouCanEat_Options.MainFrame_posX = x
	AllYouCanEat_Options.MainFrame_posY = y
	AllYouCanEat_Options.MainFrame_relP = relativePoint
	AllYouCanEat_Options.MainFrame_width = math_floor(GVAR.MainFrame:GetWidth() + 0.5)
	AllYouCanEat_Options.MainFrame_height = math_floor(GVAR.MainFrame:GetHeight() + 0.5)
end

function AllYouCanEat:Search()
	SearchQuery = GVAR.SelectionFrame.SearchBox.EditBox:GetText()
	if SearchQuery == "" then
		SearchQuery = nil
	end
	if SearchQuery then
		GVAR.SelectionFrame.SearchBox.EditBoxClear:Enable()
	else
		GVAR.SelectionFrame.SearchBox.EditBoxClear:Disable()
	end
	AllYouCanEat:UpdateData()
end

function AllYouCanEat:SearchClear()
	SearchQuery = nil
	GVAR.SelectionFrame.SearchBox.EditBox:SetText("")
	GVAR.SelectionFrame.SearchBox.EditBox:ClearFocus()
end

function AllYouCanEat:MoverInfo(action, where)
	if not action then
		GVAR.MoverInfoTopFrame:Hide()
		GVAR.MoverInfoBottomFrame:Hide()
		return
	end

	if action == "MOVE" then
		GVAR.MoverInfoTopFrameText:SetText(L["click & move"])
		GVAR.MoverInfoBottomFrameText:SetText(L["click & move"])
	else
		GVAR.MoverInfoTopFrameText:SetText(L["click & resize"])
		GVAR.MoverInfoBottomFrameText:SetText(L["click & resize"])
	end

	if where == "TOP" then
		GVAR.MoverInfoTopFrame:Show()
	else
		GVAR.MoverInfoBottomFrame:Show()
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:UpdateData()
	AllYouCanEat:CreateGUIdata()
	AllYouCanEat:FillList()
	AllYouCanEat:UpdateStatus()

	if GUI_Detail == 1 then
		TEMPLATE.SetTabButton(GVAR.SelectionFrame.ButtonAll, true)
		TEMPLATE.SetTabButton(GVAR.SelectionFrame.ButtonComplete, nil)
		TEMPLATE.SetTabButton(GVAR.SelectionFrame.ButtonIncomplete, nil)
	elseif GUI_Detail == 2 then
		TEMPLATE.SetTabButton(GVAR.SelectionFrame.ButtonAll, nil)
		TEMPLATE.SetTabButton(GVAR.SelectionFrame.ButtonComplete, true)
		TEMPLATE.SetTabButton(GVAR.SelectionFrame.ButtonIncomplete, nil)
	elseif GUI_Detail == 3 then
		TEMPLATE.SetTabButton(GVAR.SelectionFrame.ButtonAll, nil)
		TEMPLATE.SetTabButton(GVAR.SelectionFrame.ButtonComplete, nil)
		TEMPLATE.SetTabButton(GVAR.SelectionFrame.ButtonIncomplete, true)
	end
end

function AllYouCanEat:UpdateSelectionButtons()
	for i = 1, #StatisticID do
		if GUI_Selection == StatisticID[i].id then
			GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].IconBorder:Show()
			GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].Icon:SetAlpha(1)
		else
			GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].IconBorder:Hide()
			GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].Icon:SetAlpha(0.7)
		end
	end
end

function AllYouCanEat:UpdateStatusText()
	if GUI_MainDataGUID == playerGUID then
		GVAR.StatusFrameText:SetText(L["Latest Update"]..": "..GUI_scantime)
	else
		local updateTime = UNKNOWN
		if AllYouCanEat_Data.chars and
		   AllYouCanEat_Data.chars[playerRealm] and
		   AllYouCanEat_Data.chars[playerRealm][GUI_MainDataGUID] and
		   AllYouCanEat_Data.chars[playerRealm][GUI_MainDataGUID].update
		then
			updateTime = AllYouCanEat_Data.chars[playerRealm][GUI_MainDataGUID].update
		end
		GVAR.StatusFrameText:SetText(L["Latest Update"]..": "..updateTime)
	end
end

function AllYouCanEat:UpdateStatus()
	if GUI_MainDataGUID == playerGUID then
		for i = 1, #StatisticID do
			GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].TextCompleted:SetText(DATA[StatisticID[i].id].totalcompleted)
			GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].TextTotal:SetText("("..DATA[StatisticID[i].id].total..")")
		end
	else
		for i = 1, #StatisticID do
			if not GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].TextCompleted[GUI_MainDataGUID] then
				local x = 0
				for k, v in pairs(DATA[StatisticID[i].id]) do
					if k ~= "total" and k ~= "totalcompleted" then
					
						if AllYouCanEat_Data.stats and
						   AllYouCanEat_Data.stats[StatisticID[i].id] and
						   AllYouCanEat_Data.stats[StatisticID[i].id][k] and
						   AllYouCanEat_Data.stats[StatisticID[i].id][k][GUI_MainDataGUID] and
						   AllYouCanEat_Data.stats[StatisticID[i].id][k][GUI_MainDataGUID] == 1
						then
							x = x + 1
						end

					end
				end
				GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].TextCompleted[GUI_MainDataGUID] = x
			end
		end
		for i = 1, #StatisticID do
			GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].TextCompleted:SetText(GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].TextCompleted[GUI_MainDataGUID])
			GVAR.SelectionFrame.StatisticButton[StatisticID[i].id].TextTotal:SetText("("..DATA[StatisticID[i].id].total..")")
		end
	end
	AllYouCanEat:UpdateStatusText()
end

function AllYouCanEat:CreateGUIdata()
	table.wipe(GUI_Data)

	if GUI_MainDataGUID == playerGUID then

		if GUI_Detail == 1 then
			local x = 1
			for k, v in pairs(DATA[GUI_Selection]) do
				if k ~= "total" and k ~= "totalcompleted" then
					GUI_Data[x] = {}
					GUI_Data[x].itemnumber = x
					GUI_Data[x].itemid = k
					GUI_Data[x].itemlink = v.itemlink
					GUI_Data[x].bopitem = v.bopitem
					GUI_Data[x].questitem = v.questitem
					GUI_Data[x].itemrarity = v.itemrarity
					GUI_Data[x].itemname = v.itemname
					GUI_Data[x].completed = v.completed
					x = x + 1
				end
			end
		elseif GUI_Detail == 2 then
			local x = 1
			for k, v in pairs(DATA[GUI_Selection]) do
				if k ~= "total" and k ~= "totalcompleted" and v.completed then
					GUI_Data[x] = {}
					GUI_Data[x].itemnumber = x
					GUI_Data[x].itemid = k
					GUI_Data[x].itemlink = v.itemlink
					GUI_Data[x].bopitem = v.bopitem
					GUI_Data[x].questitem = v.questitem
					GUI_Data[x].itemrarity = v.itemrarity
					GUI_Data[x].itemname = v.itemname
					GUI_Data[x].completed = v.completed
					x = x + 1
				end
			end
		elseif GUI_Detail == 3 then
			local x = 1
			for k, v in pairs(DATA[GUI_Selection]) do
				if k ~= "total" and k ~= "totalcompleted" and not v.completed then
					GUI_Data[x] = {}
					GUI_Data[x].itemnumber = x
					GUI_Data[x].itemid = k
					GUI_Data[x].itemlink = v.itemlink
					GUI_Data[x].bopitem = v.bopitem
					GUI_Data[x].questitem = v.questitem
					GUI_Data[x].itemrarity = v.itemrarity
					GUI_Data[x].itemname = v.itemname
					GUI_Data[x].completed = v.completed
					x = x + 1
				end
			end
		end

	else

		if GUI_Detail == 1 then
			local x = 1
			for k, v in pairs(DATA[GUI_Selection]) do
				if k ~= "total" and k ~= "totalcompleted" then
					GUI_Data[x] = {}
					GUI_Data[x].itemnumber = x
					GUI_Data[x].itemid = k
					GUI_Data[x].itemlink = v.itemlink
					GUI_Data[x].bopitem = v.bopitem
					GUI_Data[x].questitem = v.questitem
					GUI_Data[x].itemrarity = v.itemrarity
					GUI_Data[x].itemname = v.itemname
					if AllYouCanEat_Data.stats and
					   AllYouCanEat_Data.stats[GUI_Selection] and
					   AllYouCanEat_Data.stats[GUI_Selection][k] and
					   AllYouCanEat_Data.stats[GUI_Selection][k][GUI_MainDataGUID] and
					   AllYouCanEat_Data.stats[GUI_Selection][k][GUI_MainDataGUID] == 0
					then
						GUI_Data[x].completed = false
					else
						GUI_Data[x].completed = true
					end
					x = x + 1
				end
			end
		elseif GUI_Detail == 2 then
			local x = 1
			for k, v in pairs(DATA[GUI_Selection]) do
				if k ~= "total" and k ~= "totalcompleted" then
					
					if AllYouCanEat_Data.stats and
					   AllYouCanEat_Data.stats[GUI_Selection] and
					   AllYouCanEat_Data.stats[GUI_Selection][k] and
					   AllYouCanEat_Data.stats[GUI_Selection][k][GUI_MainDataGUID] and
					   AllYouCanEat_Data.stats[GUI_Selection][k][GUI_MainDataGUID] == 1
					then
						GUI_Data[x] = {}
						GUI_Data[x].itemnumber = x
						GUI_Data[x].itemid = k
						GUI_Data[x].itemlink = v.itemlink
						GUI_Data[x].bopitem = v.bopitem
						GUI_Data[x].questitem = v.questitem
						GUI_Data[x].itemrarity = v.itemrarity
						GUI_Data[x].itemname = v.itemname
						GUI_Data[x].completed = comp
						x = x + 1
					end

				end
			end
		elseif GUI_Detail == 3 then
			local x = 1
			for k, v in pairs(DATA[GUI_Selection]) do
				if k ~= "total" and k ~= "totalcompleted" then
					
					if AllYouCanEat_Data.stats and
					   AllYouCanEat_Data.stats[GUI_Selection] and
					   AllYouCanEat_Data.stats[GUI_Selection][k] and
					   AllYouCanEat_Data.stats[GUI_Selection][k][GUI_MainDataGUID] and
					   AllYouCanEat_Data.stats[GUI_Selection][k][GUI_MainDataGUID] == 0
					then
						GUI_Data[x] = {}
						GUI_Data[x].itemnumber = x
						GUI_Data[x].itemid = k
						GUI_Data[x].itemlink = v.itemlink
						GUI_Data[x].bopitem = v.bopitem
						GUI_Data[x].questitem = v.questitem
						GUI_Data[x].itemrarity = v.itemrarity
						GUI_Data[x].itemname = v.itemname
						GUI_Data[x].completed = comp
						x = x + 1
					end

				end
			end
		end

	end

	GUI_Data_Count = #GUI_Data

	local sortfunc
	if GUI_SortBy == 1 then -- 1 = number
		if GUI_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.itemnumber < b.itemnumber then return true end
			end
		else
			sortfunc = function(a, b)
				if a.itemnumber > b.itemnumber then return true end
			end
		end
	elseif GUI_SortBy == 2 then -- 2 = itemid
		if GUI_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.itemid < b.itemid then return true end
			end
		else
			sortfunc = function(a, b)
				if a.itemid > b.itemid then return true end
			end
		end
	elseif GUI_SortBy == 3 then -- 3 = itemlink
		if GUI_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.itemlink < b.itemlink then return true end
			end
		else
			sortfunc = function(a, b)
				if a.itemlink > b.itemlink then return true end
			end
		end
	elseif GUI_SortBy == 4 then -- 4 = bopitem
		if GUI_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.bopitem < b.bopitem then return true end
			end
		else
			sortfunc = function(a, b)
				if a.bopitem > b.bopitem then return true end
			end
		end
	elseif GUI_SortBy == 5 then -- 5 = questitem
		if GUI_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.questitem < b.questitem then return true end
			end
		else
			sortfunc = function(a, b)
				if a.questitem > b.questitem then return true end
			end
		end
	elseif GUI_SortBy == 6 then -- 6 = itemname
		if GUI_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.itemname < b.itemname then return true end
			end
		else
			sortfunc = function(a, b)
				if a.itemname > b.itemname then return true end
			end
		end
	end

	table.sort(GUI_Data, sortfunc)

	-- search
	if SearchQuery then
		local newtemp = {}
		local x = 1
		local lowerSearchQuery = string_lower(SearchQuery)
		for i = 1, GUI_Data_Count do
			if string_find(string_lower(GUI_Data[i].itemname), lowerSearchQuery, 1, true) then
				newtemp[x] = {}
				newtemp[x].itemnumber = GUI_Data[i].itemnumber
				newtemp[x].itemid     = GUI_Data[i].itemid
				newtemp[x].itemlink   = GUI_Data[i].itemlink
				newtemp[x].bopitem    = GUI_Data[i].bopitem
				newtemp[x].questitem  = GUI_Data[i].questitem
				newtemp[x].itemrarity = GUI_Data[i].itemrarity
				newtemp[x].itemname   = GUI_Data[i].itemname
				newtemp[x].completed  = GUI_Data[i].completed
				x = x + 1
			end
		end

		table.wipe(GUI_Data)
		for i = 1, #newtemp do
			GUI_Data[i] = newtemp[i]
		end
		table.wipe(newtemp)
		GUI_Data_Count = #GUI_Data
	end
end

function AllYouCanEat:GUIsort(sortBy)
	if GUI_SortBy == sortBy then
		if GUI_SortOrder == 1 then
			GUI_SortOrder = 0
		else
			GUI_SortOrder = 1
		end
	else
		GUI_SortOrder = 1
	end
	GUI_SortBy = sortBy

	AllYouCanEat_Options.Data_SortBy = GUI_SortBy
	AllYouCanEat_Options.Data_SortOrder = GUI_SortOrder

	AllYouCanEat:UpdateSortButtons()
	AllYouCanEat:CreateGUIdata()
	AllYouCanEat:FillList()
end

function AllYouCanEat:UpdateSortButtons()
	local arrow
	for i = 1, 6 do
		if i == 1 then     arrow = GVAR.DataFrame.Sort.NumberSort.arrow
		elseif i == 2 then arrow = GVAR.DataFrame.Sort.ItemIdSort.arrow
		elseif i == 3 then arrow = GVAR.DataFrame.Sort.ItemLinkSort.arrow
		elseif i == 4 then arrow = GVAR.DataFrame.Sort.BopItemSort.arrow
		elseif i == 5 then arrow = GVAR.DataFrame.Sort.QuestItemSort.arrow
		elseif i == 6 then arrow = GVAR.DataFrame.Sort.ItemNameSort.arrow end
		if GUI_SortBy == i then
			if GUI_SortOrder == 1 then arrow:SetTexCoord(0, 0.5625, 0, 1) end
			if GUI_SortOrder == 0 then arrow:SetTexCoord(0.5625, 0, 1, 0) end
		else
			arrow:SetTexCoord(0, 0, 0, 0)
		end
	end
end

function AllYouCanEat:OnVerticalScroll(value)
	GVAR.DataFrame.ScrollFrame.ScrollBar:SetValue(value)
	GVAR.DataFrame.ScrollFrame.offset = math_floor((value / 12) + 0.5)
	AllYouCanEat:FillList()
end

function AllYouCanEat:FillList()
	local scrollFrame = GVAR.DataFrame.ScrollFrame
	local scrollFrameHeight = 0
	local numToDisplay = GUI_Data_curNum
	local valueStep = 12

	if GUI_Data_Count < numToDisplay then
		numToDisplay = GUI_Data_Count
	end

	if GUI_Data_Count > 0 then
		scrollFrameHeight = (GUI_Data_Count - numToDisplay) * valueStep
		if scrollFrameHeight < 0 then
			scrollFrameHeight = 0
		end
	end
	scrollFrame.ScrollBar:SetMinMaxValues(0, scrollFrameHeight)
	scrollFrame.ScrollBar:SetValueStep(valueStep)

	if GUI_Data_Count <= GUI_Data_curNum then
		scrollFrame.ScrollBarScrollUpButton:Disable()
		scrollFrame.ScrollBarScrollDownButton:Disable()
		scrollFrame.ScrollBarScrollUpButton:SetAlpha(0.2)
		scrollFrame.ScrollBarScrollDownButton:SetAlpha(0.2)
		scrollFrame.ScrollBarThumbTexture:SetAlpha(0.2)
	else
		scrollFrame.ScrollBarScrollUpButton:Enable()
		scrollFrame.ScrollBarScrollDownButton:Enable()
		scrollFrame.ScrollBarScrollUpButton:SetAlpha(1)
		scrollFrame.ScrollBarScrollDownButton:SetAlpha(1)
		scrollFrame.ScrollBarThumbTexture:SetAlpha(1)
	end

	if scrollFrame.ScrollBar:GetValue() == 0 then
		scrollFrame.ScrollBarScrollUpButton:Disable()
	else
		scrollFrame.ScrollBarScrollUpButton:Enable()
	end
	if ((scrollFrame.ScrollBar:GetValue() - scrollFrameHeight) == 0) then
		scrollFrame.ScrollBarScrollDownButton:Disable()
	else
		scrollFrame.ScrollBarScrollDownButton:Enable()
	end

	for i=1, GUI_Data_maxNum do
		GVAR.DataFrame.ScrollFrameButton[i]:Hide()
	end

	local j = 1
	local sliderPos = scrollFrame.offset
	for i=sliderPos+1, sliderPos+numToDisplay do
		if sliderPos < GUI_Data_Count then
			GVAR.DataFrame.ScrollFrameButton[j].NumberButton:SetText(GUI_Data[i].itemnumber)
			GVAR.DataFrame.ScrollFrameButton[j].ItemIdButton:SetText("|cff"..itemQuality[ GUI_Data[i].itemrarity ].color..GUI_Data[i].itemid.."|r")
			GVAR.DataFrame.ScrollFrameButton[j].ItemNameButton:SetText(GUI_Data[i].itemname)

			if GUI_Data[i].completed then
				GVAR.DataFrame.ScrollFrameButton[j].ItemNameButton:SetTextColor(0, 0.75, 0, 1)
			else
				GVAR.DataFrame.ScrollFrameButton[j].ItemNameButton:SetTextColor(1, 0, 0, 1)
			end

			if GUI_Data[i].itemlink == 1 then
				GVAR.DataFrame.ScrollFrameButton[j].ItemLinkButton:SetTexCoord(0.125, 0.71875, 0.125, 0.375) -- (4/32, 23/32, 4/32, 12/32)
			else
				GVAR.DataFrame.ScrollFrameButton[j].ItemLinkButton:SetTexCoord(0, 0, 0, 0)
			end

			if GUI_Data[i].bopitem == 1 then
				GVAR.DataFrame.ScrollFrameButton[j].BopItemButton:SetText(L["BoP"])
			else
				GVAR.DataFrame.ScrollFrameButton[j].BopItemButton:SetText("")
			end

			if GUI_Data[i].questitem == 1 then
				GVAR.DataFrame.ScrollFrameButton[j].QuestItemButton:SetText(L["Q"])
			else
				GVAR.DataFrame.ScrollFrameButton[j].QuestItemButton:SetText("")
			end

			GVAR.DataFrame.ScrollFrameButton[j]:Show()
			GVAR.DataFrame.ScrollFrameButton[j].itemID = GUI_Data[i].itemid
			j = j + 1
		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:InitialStatisticScan()
	for i = 1, #StatisticID do
		local statID = StatisticID[i].id
		local total = 0
		local totalcompleted = 0

		if not DATA[statID] then DATA[statID] = {} end

		-- TODO need redo if Blizzard fix num bug from statID 1774 and 1775 -- START
		--print("GetAchievementInfo:", GetAchievementInfo(statID))
		local num = GetAchievementNumCriteria(statID)
		--print("-----", i, "#", StatisticID[i].name, "num1:", num)
		if statID == 1774 then
			num = 221
		elseif statID == 1775 then
			num = 513
		end
		--print("-----", i, "#", StatisticID[i].name, "num2:", num)
		-- TODO need redo if Blizzard fix num bug from statID 1774 and 1775 -- END

		for j = 1, num do
			local itemName, _, _, _, _, _, _, itemID = GetAchievementCriteriaInfo(statID, j)
			if not itemID then break end
			local _, _, isCompleted = GetAchievementCriteriaInfo(statID, j)
			--print(j, "#", statID, itemName, itemID, "|-->", isCompleted)

			if not DATA[statID][itemID] then
				DATA[statID][itemID] = {}
				DATA[statID][itemID].itemlink   = 0
				DATA[statID][itemID].bopitem    = 0
				DATA[statID][itemID].questitem  = 0
				DATA[statID][itemID].itemrarity = 9
				DATA[statID][itemID].itemname   = itemName
				DATA[statID][itemID].completed  = isCompleted
				DATA[statID][itemID].criteriaNUM = j

				total = total + 1

				if isCompleted then
					totalcompleted = totalcompleted + 1
				end
			end

		end

		DATA[statID].total = total
		DATA[statID].totalcompleted = totalcompleted
	end
	UpdateTime()
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function AllYouCanEat_Hook_UseContainerItem(bag, slot)
	if not bag or (bag and type(bag) ~= "number") then return end
	if bag == BANK_CONTAINER then return end
	if not slot or (slot and type(slot) ~= "number") then return end

	local itemID = GetContainerItemID(bag, slot)
	if not itemID then return end

	for i = 1, #StatisticID do
		local statID = StatisticID[i].id
		if DATA[statID][itemID] and not DATA[statID][itemID].completed then
			UPDATE_itemID = itemID
			UPDATE_statID = statID
			UPDATE_catID  = i
			criteriaTimer = GetTime()
			--AllYouCanEat:RegisterEvent("CRITERIA_UPDATE") -- print("register CRITERIA_UPDATE") -- TODO: test
			return
		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function OnEvent(self, event)
	if event == "CRITERIA_UPDATE" then
		if not UPDATE_itemID then return end -- <- !!! muy importante, amigo !!!

		local _, _, isCompleted = GetAchievementCriteriaInfo(UPDATE_statID, DATA[UPDATE_statID][UPDATE_itemID].criteriaNUM)

		if isCompleted then
			DATA[UPDATE_statID][UPDATE_itemID].completed = true
			DATA[UPDATE_statID].totalcompleted = DATA[UPDATE_statID].totalcompleted + 1

			if AllYouCanEat_Data.stats and
			   AllYouCanEat_Data.stats[UPDATE_statID] and
			   AllYouCanEat_Data.stats[UPDATE_statID][UPDATE_itemID] and
			   AllYouCanEat_Data.stats[UPDATE_statID][UPDATE_itemID][playerGUID]
			then
				AllYouCanEat_Data.stats[UPDATE_statID][UPDATE_itemID][playerGUID] = 1
			end

			local _, itemLink = GetItemInfo(UPDATE_itemID)
			if itemLink then
				Print("|cffff9900"..NEW.."|r", itemLink, "- |cffcccccc("..StatisticID[UPDATE_catID].name.." | "..L["ItemID"]..": "..UPDATE_itemID..")|r")
			else
				Print("|cffff9900"..NEW.."|r", DATA[UPDATE_statID][UPDATE_itemID].itemname, "- |cffcccccc("..StatisticID[UPDATE_catID].name.." | "..L["ItemID"]..": "..UPDATE_itemID..")|r")
			end

			-- if UPDATE_itemID exists more than one time START
			for i = 1, #StatisticID do
				local statID = StatisticID[i].id
				if DATA[statID][UPDATE_itemID] and not DATA[statID][UPDATE_itemID].completed then

					DATA[statID][UPDATE_itemID].completed = true
					DATA[statID].totalcompleted = DATA[statID].totalcompleted + 1

					if AllYouCanEat_Data.stats and
					   AllYouCanEat_Data.stats[statID] and
					   AllYouCanEat_Data.stats[statID][UPDATE_itemID] and
					   AllYouCanEat_Data.stats[statID][UPDATE_itemID][playerGUID]
					then
						AllYouCanEat_Data.stats[statID][UPDATE_itemID][playerGUID] = 1
					end

					if itemLink then
						Print("|cffff9900"..NEW.."|r", itemLink, "- |cffcccccc("..StatisticID[i].name.." | "..L["ItemID"]..": "..UPDATE_itemID..")|r")
					else
						Print("|cffff9900"..NEW.."|r", DATA[statID][UPDATE_itemID].itemname, "- |cffcccccc("..StatisticID[i].name.." | "..L["ItemID"]..": "..UPDATE_itemID..")|r")
					end

				end
			end
			-- if UPDATE_itemID exists more than one time END

			UpdateTime()
			if GVAR.MainFrame:IsShown() then
				AllYouCanEat:UpdateData()
			end

			UPDATE_itemID = nil
			--AllYouCanEat:UnregisterEvent("CRITERIA_UPDATE") -- print("un-register CRITERIA_UPDATE") -- TODO: test

		else
			-- the duration between a successful item click (with itemID from initial stats scan) and any CRITERIA_UPDATE action is set to 5 seconds
			if GetTime() - criteriaTimer > 5 then
				UPDATE_itemID = nil
				--AllYouCanEat:UnregisterEvent("CRITERIA_UPDATE") -- print("un-register CRITERIA_UPDATE time") -- TODO: test
			end
		end
	elseif event == "PLAYER_LOGIN" then

		AllYouCanEat:InitOptions()

		playerRealm   = GetCVar("realmName")
		playerName    = UnitName("player")
		playerGUID    = UnitGUID("player")
		playerFaction = UnitFactionGroup("player")
		playerClass   = select(2, UnitClass("player"))

		GUI_MainDataGUID = playerGUID

		AllYouCanEat:SaveCharacterData()

		for k in pairs(AllYouCanEat_Data.chars[playerRealm]) do
			GUI_CharListData_maxNum = GUI_CharListData_maxNum + 1
		end

		AllYouCanEat:InitialStatisticScan()
		AllYouCanEat:CheckSavedStatisticData()
		AllYouCanEat:CreateInterfaceOptions()
		AllYouCanEat:LDBcheck()
		AllYouCanEat:CreateMinimapButton()
		AllYouCanEat:CreateMainFrame()
		AllYouCanEat:InitializeFrame()
		table.insert(UISpecialFrames, "AllYouCanEat_MainFrame")

		AllYouCanEat:CheckIfTooltipCodeIsNeeded()

		InstallHooks(_G.GameTooltip, Hooks)
		InstallHooks(_G.ItemRefTooltip, Hooks)

		hooksecurefunc("UseContainerItem", AllYouCanEat_Hook_UseContainerItem)

		AllYouCanEat:UnregisterEvent("PLAYER_LOGIN")
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

AllYouCanEat:RegisterEvent("CRITERIA_UPDATE") -- TODO: find better way
AllYouCanEat:RegisterEvent("PLAYER_LOGIN")
AllYouCanEat:SetScript("OnEvent", OnEvent)